"""ASN.1 object identifiers. This module contains a dictionary of known OIDs.

See http://luca.ntop.org/Teaching/Appunti/asn1.html for a good introduction
to ASN.1.
"""

from smime.crypto import error
from smime.crypto.asn1 import tag
from smime.crypto.asn1 import types


@types.Universal(6, tag.PRIMITIVE)
class ObjectIdentifier(types.Simple):
    """Object identifier."""

    def _name(self, dict_idx):
        try:
            return _OID_NAME_DICT[self][dict_idx]
        except KeyError:
            # fall back to OID
            return ".".join(map(str, self._value))

    @property
    def short_name(self):
        """Return the short name representation of an OID."""
        return self._name(1)

    @property
    def long_name(self):
        """Return the long name representation of an OID."""
        return self._name(0)

    def __str__(self):
        return self.short_name

    @property
    def value(self):
        """The value of an OID is a tuple of integers."""
        return self._value

    @staticmethod
    def _encode_component(value):
        """Encode an OID component as a bytearray.

        Args:
            value: an integer component value

        Returns:
            a bytearray representing the encoded component.
        """
        int_bytes = bytearray()
        # Encode in base-128.
        # All bytes apart from the lsb have the high bit set.
        int_bytes.append(value & 0x7f)
        value >>= 7
        while value:
            int_bytes.append(value & 0x7f | 0x80)
            value >>= 7
        int_bytes.reverse()
        return int_bytes

    @classmethod
    def _read_component(cls, int_bytes):
        """Parse a single component from a non-empty bytearray.

        Args:
            int_bytes: a non-empty bytearray.

        Returns:
            a (component, rest) tuple with the decoded integer and the
                remaining bytes of the bytearray.
        """
        ret = 0
        i = 0
        while int_bytes[i] & 0x80:
            num = int_bytes[i] & 0x7f
            if not ret and not num:
                # The component must be encoded with as few digits as possible,
                # i.e., leading zeroes are not allowed. Since ASN.1 libraries
                # interpret leading 0x80-octets differently, this may be
                # indicative of an attempt to trick a browser into accepting a
                # certificate it shouldn't. See page 7 of
                # www.cosic.esat.kuleuven.be/publications/article-1432.pdf
                raise error.ASN1Error("Leading 0x80 octets in the base-128 "
                                      "encoding of  OID component")
            ret |= num
            ret <<= 7
            i += 1

        ret |= int_bytes[i]
        return ret, int_bytes[i+1:]

    def _encode_value(self):
        int_bytes = bytearray()
        # ASN.1 specifies that the first two components are encoded together
        # as c0*40 + c1.
        int_bytes += self._encode_component(self._value[0]*40 + self._value[1])
        for v in self._value[2:]:
            int_bytes += self._encode_component(v)
        return str(int_bytes)

    @classmethod
    def _convert_value(cls, value):
        if isinstance(value, ObjectIdentifier):
            return value.value
        else:
            if isinstance(value, str):
                value = [int(v) for v in value.split(".")]
            if len(value) < 2:
                raise ValueError("OID must have at least 2 components")
            if not all([v >= 0 for v in value]):
                raise ValueError("OID cannot have negative components")
            if value[0] > 2:
                raise ValueError("First OID component must be 0, 1 or 2, "
                                 "got %d" % value[0])
            if value[0] <= 1 and value[1] > 39:
                raise ValueError("Second OID component must be <= 39 if "
                                 "first component is <= 1; got %d, %d" %
                                 (value[0], value[1]))
            return tuple(value)

    @classmethod
    def _decode_value(cls, buf, strict=True):
        """Decode from a string or string buffer."""
        if buf in _OID_DECODING_DICT:
            return _OID_DECODING_DICT[buf]
        if not buf:
            raise error.ASN1Error("Invalid encoding")
        int_bytes = bytearray(buf)

        # Last byte can't have the high bit set.
        if int_bytes[-1] & 0x80:
            raise error.ASN1Error("Invalid encoding")

        components = []

        first, int_bytes = cls._read_component(int_bytes)
        if first < 40:
            components += [0, first]
        elif first < 80:
            components += [1, first - 40]
        else:
            components += [2, first - 80]

        while int_bytes:
            component, int_bytes = cls._read_component(int_bytes)
            components.append(component)

        return tuple(components)

###############################################################################
#                           Known object identifiers                          #
#                                                                             #
# If you add a new OID, make sure to also add its commonly known alias to     #
# _OID_NAME_DICT.                                                             #
###############################################################################

# Signature and public key algorithms
# RFC 3279
RSA_ENCRYPTION = ObjectIdentifier(value="1.2.840.113549.1.1.1")
MD2_WITH_RSA_ENCRYPTION = ObjectIdentifier(value="1.2.840.113549.1.1.2")
MD5_WITH_RSA_ENCRYPTION = ObjectIdentifier(value="1.2.840.113549.1.1.4")
SHA1_WITH_RSA_ENCRYPTION = ObjectIdentifier(value="1.2.840.113549.1.1.5")
ID_DSA = ObjectIdentifier(value="1.2.840.10040.4.1")
ID_DSA_WITH_SHA1 = ObjectIdentifier(value="1.2.840.10040.4.3")
ID_EC_PUBLICKEY = ObjectIdentifier(value="1.2.840.10045.2.1")
ECDSA_WITH_SHA1 = ObjectIdentifier(value="1.2.840.10045.4.1")
# RFC 5758
ECDSA_WITH_SHA224 = ObjectIdentifier(value="1.2.840.10045.4.3.1")
ECDSA_WITH_SHA256 = ObjectIdentifier(value="1.2.840.10045.4.3.2")
ECDSA_WITH_SHA384 = ObjectIdentifier(value="1.2.840.10045.4.3.3")
ECDSA_WITH_SHA512 = ObjectIdentifier(value="1.2.840.10045.4.3.4")
# RFC 4055
ID_RSASSA_PSS = ObjectIdentifier(value="1.2.840.113549.1.1.10")
ID_SHA256_WITH_RSA_ENCRYPTION = ObjectIdentifier(value="1.2.840.113549.1.1.11")
ID_SHA384_WITH_RSA_ENCRYPTION = ObjectIdentifier(value="1.2.840.113549.1.1.12")
ID_SHA512_WITH_RSA_ENCRYPTION = ObjectIdentifier(value="1.2.840.113549.1.1.13")
ID_SHA224_WITH_RSA_ENCRYPTION = ObjectIdentifier(value="1.2.840.113549.1.1.14")
# RFC 4491
ID_GOSTR3411_94_WITH_GOSTR3410_94 = ObjectIdentifier(value="1.2.643.2.2.4")
ID_GOSTR3411_94_WITH_GOSTR3410_2001 = ObjectIdentifier(value="1.2.643.2.2.3")
# RFC 5758
ID_DSA_WITH_SHA224 = ObjectIdentifier(value="2.16.840.1.101.3.4.3.1")
ID_DSA_WITH_SHA256 = ObjectIdentifier(value="2.16.840.1.101.3.4.3.2")
# Naming attributes (RFC 5280)
ID_AT_NAME = ObjectIdentifier(value="2.5.4.41")
ID_AT_SURNAME = ObjectIdentifier(value="2.5.4.4")
ID_AT_GIVEN_NAME = ObjectIdentifier(value="2.5.4.42")
ID_AT_INITIALS = ObjectIdentifier(value="2.5.4.43")
ID_AT_GENERATION_QUALIFIER = ObjectIdentifier(value="2.5.4.44")
ID_AT_COMMON_NAME = ObjectIdentifier(value="2.5.4.3")
ID_AT_LOCALITY_NAME = ObjectIdentifier(value="2.5.4.7")
ID_AT_STATE_OR_PROVINCE_NAME = ObjectIdentifier(value="2.5.4.8")
ID_AT_ORGANIZATION_NAME = ObjectIdentifier(value="2.5.4.10")
ID_AT_ORGANIZATIONAL_UNIT_NAME = ObjectIdentifier(value="2.5.4.11")
ID_AT_TITLE = ObjectIdentifier(value="2.5.4.12")
ID_AT_DN_QUALIFIER = ObjectIdentifier(value="2.5.4.46")
ID_AT_COUNTRY_NAME = ObjectIdentifier(value="2.5.4.6")
ID_AT_SERIAL_NUMBER = ObjectIdentifier(value="2.5.4.5")
ID_AT_PSEUDONYM = ObjectIdentifier(value="2.5.4.65")
ID_DOMAIN_COMPONENT = ObjectIdentifier(value="0.9.2342.19200300.100.1.25")
ID_EMAIL_ADDRESS = ObjectIdentifier(value="1.2.840.113549.1.9.1")

# Other naming attributes commonly found in certs
ID_AT_STREET_ADDRESS = ObjectIdentifier(value="2.5.4.9")
ID_AT_DESCRIPTION = ObjectIdentifier(value="2.5.4.13")
ID_AT_BUSINESS_CATEGORY = ObjectIdentifier(value="2.5.4.15")
ID_AT_POSTAL_CODE = ObjectIdentifier(value="2.5.4.17")
ID_AT_POST_OFFICE_BOX = ObjectIdentifier(value="2.5.4.18")

# Standard X509v3 certificate extensions
ID_CE_AUTHORITY_KEY_IDENTIFIER = ObjectIdentifier(value="2.5.29.35")
ID_CE_SUBJECT_KEY_IDENTIFIER = ObjectIdentifier(value="2.5.29.14")
ID_CE_KEY_USAGE = ObjectIdentifier(value="2.5.29.15")
ID_CE_PRIVATE_KEY_USAGE_PERIOD = ObjectIdentifier(value="2.5.29.16")
ID_CE_CERTIFICATE_POLICIES = ObjectIdentifier(value="2.5.29.32")
ID_CE_SUBJECT_ALT_NAME = ObjectIdentifier(value="2.5.29.17")
ID_CE_ISSUER_ALT_NAME = ObjectIdentifier(value="2.5.29.18")
ID_CE_SUBJECT_DIRECTORY_ATTRIBUTES = ObjectIdentifier(value="2.5.29.9")
ID_CE_BASIC_CONSTRAINTS = ObjectIdentifier(value="2.5.29.19")
ID_CE_NAME_CONSTRAINTS = ObjectIdentifier(value="2.5.29.30")
ID_CE_POLICY_CONSTRAINTS = ObjectIdentifier(value="2.5.29.30")
ID_CE_EXT_KEY_USAGE = ObjectIdentifier(value="2.5.29.37")
ID_CE_CRL_DISTRIBUTION_POINTS = ObjectIdentifier(value="2.5.29.31")
ID_CE_INHIBIT_ANY_POLICY = ObjectIdentifier(value="2.5.29.54")
ID_PE_AUTHORITY_INFO_ACCESS = ObjectIdentifier(value="1.3.6.1.5.5.7.1.1")
ID_PE_SUBJECT_INFO_ACCESS = ObjectIdentifier(value="1.3.6.1.5.5.7.1.11")

# RFC 3280 - Used in ExtendedKeyUsage extension
ID_KP_SERVER_AUTH = ObjectIdentifier(value="1.3.6.1.5.5.7.3.1")
ID_KP_CLIENT_AUTH = ObjectIdentifier(value="1.3.6.1.5.5.7.3.2")
ID_KP_CODE_SIGNING = ObjectIdentifier(value="1.3.6.1.5.5.7.3.3")
ID_KP_EMAIL_PROTECTION = ObjectIdentifier(value="1.3.6.1.5.5.7.3.4")
ID_KP_TIME_STAMPING = ObjectIdentifier(value="1.3.6.1.5.5.7.3.8")
ID_KP_OCSP_SIGNING = ObjectIdentifier(value="1.3.6.1.5.5.7.3.9")

# RFC 3280 - Used in Authority Info Access extension
ID_AD_OCSP = ObjectIdentifier(value="1.3.6.1.5.5.7.48.1")
ID_AD_CA_ISSUERS = ObjectIdentifier(value="1.3.6.1.5.5.7.48.2")

# Certificate Policy related OIDs
ID_QT_CPS = ObjectIdentifier(value="1.3.6.1.5.5.7.2.1")
ID_QT_UNOTICE = ObjectIdentifier(value="1.3.6.1.5.5.7.2.2")
ANY_POLICY = ObjectIdentifier(value="2.5.29.32.0")

# CT Specific
CT_EMBEDDED_SCT_LIST = ObjectIdentifier(value="1.3.6.1.4.1.11129.2.4.2")
CT_POISON = ObjectIdentifier(value="1.3.6.1.4.1.11129.2.4.3")
CT_PRECERTIFICATE_SIGNING = ObjectIdentifier(value="1.3.6.1.4.1.11129.2.4.4")

# CMS Specific
ID_CT_CONTENT_INFO = ObjectIdentifier(value='1.2.840.113549.1.9.16.1.6')
ID_DATA = ObjectIdentifier(value='1.2.840.113549.1.7.1')
ID_SIGNED_DATA = ObjectIdentifier(value='1.2.840.113549.1.7.2')
ID_ENVELOPED_DATA = ObjectIdentifier(value='1.2.840.113549.1.7.3')
ID_DIGESTED_DATA = ObjectIdentifier(value='1.2.840.113549.1.7.5')
ID_ENCRYPTED_DATA = ObjectIdentifier(value='1.2.840.113549.1.7.6')
ID_AUTHENTICATED_DATA = ObjectIdentifier(value='1.2.840.113549.1.9.16.1.2')
ID_CONTENT_TYPE = ObjectIdentifier(value='1.2.840.113549.1.9.3')
ID_MESSAGE_DIGEST = ObjectIdentifier(value='1.2.840.113549.1.9.4')
ID_SIGNING_TIME = ObjectIdentifier(value='1.2.840.113549.1.9.5')
ID_COUNTERSIGNATURE = ObjectIdentifier(value='1.2.840.113549.1.9.6')

# AES Specific
ID_AES128_CBC = ObjectIdentifier(value='2.16.840.1.101.3.4.1.2')
ID_AES192_CBC = ObjectIdentifier(value='2.16.840.1.101.3.4.1.22')
ID_AES256_CBC = ObjectIdentifier(value='2.16.840.1.101.3.4.1.42')


_OID_NAME_DICT = {
    # Object identifier long names taken verbatim from the RFCs.
    # Short names are colloquial.
    RSA_ENCRYPTION: ("rsaEncryption", "RSA"),
    MD2_WITH_RSA_ENCRYPTION: ("md2WithRSAEncryption", "RSA-MD2"),
    MD5_WITH_RSA_ENCRYPTION: ("md5WithRSAEncryption", "RSA-MD5"),
    SHA1_WITH_RSA_ENCRYPTION: ("sha-1WithRSAEncryption", "RSA-SHA1"),
    ID_DSA: ("id-dsa", "DSA"),
    ID_DSA_WITH_SHA1: ("id-dsa-with-sha1", "DSA-SHA1"),
    ID_EC_PUBLICKEY: ("id-ecPublicKey", "EC-PUBKEY"),
    ECDSA_WITH_SHA1: ("ecdsa-with-SHA1", "ECDSA-SHA1"),
    ECDSA_WITH_SHA224: ("ecdsa-with-SHA224", "ECDSA-SHA224"),
    ECDSA_WITH_SHA256: ("ecdsa-with-SHA256", "ECDSA-SHA256"),
    ECDSA_WITH_SHA384: ("ecdsa-with-SHA384", "ECDSA-SHA384"),
    ECDSA_WITH_SHA512: ("ecdsa-with-SHA512", "ECDSA-SHA512"),
    ID_RSASSA_PSS: ("id-RSASSA-PSS", "RSASSA-PSS"),
    ID_GOSTR3411_94_WITH_GOSTR3410_94: ("id-GostR3411-94-with-GostR3410-94",
                                        "GOST94"),
    ID_GOSTR3411_94_WITH_GOSTR3410_2001: ("id-GostR3411-94-with-GostR3410-2001",
                                          "GOST2001"),
    ID_SHA256_WITH_RSA_ENCRYPTION: ("sha256WithRSAEncryption", "RSA-SHA256"),
    ID_SHA384_WITH_RSA_ENCRYPTION: ("sha384WithRSAEncryption", "RSA-SHA384"),
    ID_SHA512_WITH_RSA_ENCRYPTION: ("sha512WithRSAEncryption", "RSA-SHA512"),
    ID_SHA224_WITH_RSA_ENCRYPTION: ("sha224WithRSAEncryption", "RSA-SHA224"),
    ID_DSA_WITH_SHA224: ("id-dsa-with-sha224", "DSA-SHA224"),
    ID_DSA_WITH_SHA256: ("id-dsa-with-sha256", "DSA-SHA256"),
    ID_AT_NAME: ("id-at-name", "name"),
    ID_AT_SURNAME: ("id-at-surname", "surname"),
    ID_AT_GIVEN_NAME: ("id-at-givenName", "givenName"),
    ID_AT_INITIALS: ("id-at-initials", "initials"),
    ID_AT_GENERATION_QUALIFIER: ("id-at-generationQualifier", "genQualifier"),
    ID_AT_COMMON_NAME: ("id-at-commonName", "CN"),
    ID_AT_LOCALITY_NAME: ("id-at-localityName", "L"),
    ID_AT_STATE_OR_PROVINCE_NAME: ("id-at-stateOrProvinceName", "ST"),
    ID_AT_ORGANIZATION_NAME: ("id-at-organizationName", "O"),
    ID_AT_ORGANIZATIONAL_UNIT_NAME: ("id-at-organizationalUnitName", "OU"),
    ID_AT_TITLE: ("id-at-title", "title"),
    ID_AT_DN_QUALIFIER: ("id-at-dnQualifier", "dnQualifier"),
    ID_AT_COUNTRY_NAME: ("id-at-countryName", "C"),
    ID_AT_SERIAL_NUMBER: ("id-at-serialNumber", "serialNumber"),
    ID_AT_PSEUDONYM: ("id-at-pseudonym", "pseudonym"),
    ID_DOMAIN_COMPONENT: ("id-domainComponent", "domainComponent"),
    ID_EMAIL_ADDRESS: ("id-emailAddress", "email"),
    ID_AT_STREET_ADDRESS: ("id-at-streetAddress", "streetAddress"),
    ID_AT_DESCRIPTION: ("id-at-description", "description"),
    ID_AT_BUSINESS_CATEGORY: ("id-at-businessCategory", "businessCategory"),
    ID_AT_POSTAL_CODE: ("id-at-postalCode", "postalCode"),
    ID_AT_POST_OFFICE_BOX: ("id-at-postOfficeBox", "postOfficeBox"),
    ID_CE_AUTHORITY_KEY_IDENTIFIER: ("id-ce-authorityKeyIdentifier",
                                     "authorityKeyIdentifier"),
    ID_CE_SUBJECT_KEY_IDENTIFIER: ("id-ce-subjectKeyIdentifier",
                                   "subjectKeyIdentifier"),
    ID_CE_KEY_USAGE: ("id-ce-keyUsage", "keyUsage"),
    ID_CE_PRIVATE_KEY_USAGE_PERIOD: ("id-ce-privateKeyUsagePeriod",
                                     "privateKeyUsagePeriod"),
    ID_CE_CERTIFICATE_POLICIES: ("id-ce-certificatePolicies",
                                 "certificatePolicies"),
    ID_CE_SUBJECT_ALT_NAME: ("id-ce-subjectAltName", "subjectAltName"),
    ID_CE_ISSUER_ALT_NAME: ("id-ce-issuerAltName", "issuerAltName"),
    ID_CE_SUBJECT_DIRECTORY_ATTRIBUTES: ("id-ce-subjectDirectoryAttributes",
                                         "subjectDirectoryAttributes"),
    ID_CE_BASIC_CONSTRAINTS: ("id-ce-basicConstraints", "basicConstraints"),
    ID_CE_NAME_CONSTRAINTS: ("id-ce-nameConstraints", "nameConstraints"),
    ID_CE_POLICY_CONSTRAINTS: ("id-ce-policyConstraints", "policyConstraints"),
    ID_CE_EXT_KEY_USAGE: ("id-ce-extKeyUsage", "extendedKeyUsage"),
    ID_CE_CRL_DISTRIBUTION_POINTS: ("id-ce-cRLDistributionPoints",
                                    "CRLDistributionPoints"),
    ID_CE_INHIBIT_ANY_POLICY: ("id-ce-inhibitAnyPolicy", "inhibitAnyPolicy"),
    ID_PE_AUTHORITY_INFO_ACCESS: ("id-pe-authorityInfoAccess",
                                  "authorityInformationAccess"),
    ID_PE_SUBJECT_INFO_ACCESS: ("id-pe-subjectInfoAccess",
                                "subjectInformationAccess"),

    ID_KP_SERVER_AUTH: ("id-kp-serverAuth", "serverAuth"),
    ID_KP_CLIENT_AUTH: ("id-kp-clientAuth", "clientAuth"),
    ID_KP_CODE_SIGNING: ("id-kp-codeSigning", "codeSigning"),
    ID_KP_EMAIL_PROTECTION: ("id-kp-emailProtection", "emailProtection"),
    ID_KP_TIME_STAMPING: ("id-kp-timeStamping", "timeStamping"),
    ID_KP_OCSP_SIGNING: ("id-kp-OCSPSigning", "OCSPSigning"),

    ID_AD_OCSP: ("id-ad-ocsp", "OCSP"),
    ID_AD_CA_ISSUERS: ("id-ad-caIssuers", "caIssuers"),
    ID_QT_CPS: ("id-qt-cps", "CPS"),
    ID_QT_UNOTICE: ("id-qt-unotice", "UserNotice"),
    ANY_POLICY: ("anyPolicy", "anyPolicy"),

    CT_EMBEDDED_SCT_LIST: ("ctEmbeddedSCT", "ctEmbeddedSCT"),
    CT_POISON: ("ctPoison", "ctPoison"),
    CT_PRECERTIFICATE_SIGNING: ("ctPrecertificateSigningCert", "ctPrecertificateSigningCert"),

    ID_CT_CONTENT_INFO: ('id-ct-contentInfo', 'contentInfo'),
    ID_DATA: ('id-data', 'data'),
    ID_SIGNED_DATA: ('id-signedData', 'signedData'),
    ID_ENVELOPED_DATA: ('id-envelopedData', 'envelopedData'),
    ID_DIGESTED_DATA: ('id-digestedData', 'digestedData'),
    ID_ENCRYPTED_DATA: ('id-encryptedData', 'encryptedData'),
    ID_AUTHENTICATED_DATA: ('id-authenticatedData', 'authenticatedData'),
    ID_CONTENT_TYPE: ('id-contentType', 'contentType'),
    ID_MESSAGE_DIGEST: ('id-messageDigest', 'messageDigest'),
    ID_SIGNING_TIME: ('id-signingTime', 'signingTime'),
    ID_COUNTERSIGNATURE: ('id-countersignature', 'countersignature'),

    ID_AES128_CBC: ('id-aes128-CBC', 'AES-128-CBC'),
    ID_AES192_CBC: ('id-aes192-CBC', 'AES-192-CBC'),
    ID_AES256_CBC: ('id-aes256-CBC', 'AES-256-CBC')
    }

_OID_DECODING_DICT = {}

for oid in _OID_NAME_DICT:
  _OID_DECODING_DICT[oid._encode_value()] = oid._value


from smime.crypto.asn1 import types
from smime.crypto.asn1 import x509_time
from smime.crypto.asn1 import cms_common


class MessageDigest(types.OctetString):
    """
    MessageDigest ::= OCTET STRING
    """
    pass


class Time(types.Choice):
    """
    Time ::= CHOICE {
        utcTime          UTCTime,
        generalizedTime  GeneralizedTime }
    """
    components = (
        (types.Component('utcTime', x509_time.UTCTime)),
        (types.Component('generalizedTime', x509_time.GeneralizedTime))
        )


class SigningTime(Time):
    """
    SigningTime ::= Time
    """
    pass


class Countersignature(cms_common.SignerInfo):
    """
    Countersignature ::= SignerInfo
    """
    pass


"""Test meta-data and configuration."""

import os

CRYPTO_TEST_DATA_DIR = "smime/crypto/testdata/"

def get_test_file_path(filename):
    return os.path.join(os.curdir, CRYPTO_TEST_DATA_DIR, filename)


#!/usr/bin/env python

import unittest

from smime.crypto.asn1 import oid
from smime.crypto.asn1 import type_test_base


class ObjectIdentifierTest(type_test_base.TypeTestBase):
    asn1_type = oid.ObjectIdentifier

    hashable = True

    initializers = (
        ((0, 0), "0.0"),
        ((1, 2), "1.2"),
        ((2, 5), "2.5"),
        ((1, 2, 3, 4), "1.2.3.4"),
        ((1, 2, 840, 113549), "1.2.840.113549"),
        ((1, 2, 840, 113549, 1), "1.2.840.113549.1"),
        )

    bad_initializers = (
        # Too short.
        ("0", ValueError),
        ((0,), ValueError),
        (("1"), ValueError),
        ((1,), ValueError),
        # Negative components.
        ("-1", ValueError),
        ((-1,), ValueError),
        ("1.2.3.-4", ValueError),
        ((1, 2, 3, -4), ValueError),
        # Invalid components.
        ("3.2.3.4", ValueError),
        ((3, 2, 3, 4), ValueError),
        ("0.40.3.4", ValueError),
        ((0, 40, 3, 4), ValueError),
        )

    encode_test_vectors = (
        # Example from ASN.1 spec.
        ("2.100.3", "0603813403"),
        # More examples.
        ("0.0", "060100"),
        ("1.2", "06012a"),
        ("2.5", "060155"),
        ("1.2.3.4", "06032a0304"),
        ("1.2.840", "06032a8648"),
        ("1.2.840.113549", "06062a864886f70d"),
        ("1.2.840.113549.1", "06072a864886f70d01")
        )

    bad_encodings = (
        # Empty OID.
        ("0600"),
        # Last byte has high bit set.
        ("06020080"),
        ("06032a86c8"),
        # Leading '80'-octets in component.
        ("06042a8086c8"),
        # Indefinite length.
        ("06808134030000")
        )

    bad_strict_encodings = ()

    def test_dictionary(self):
        rsa = oid.ObjectIdentifier(value=oid.RSA_ENCRYPTION)
        self.assertEqual("rsaEncryption", rsa.long_name)
        self.assertEqual("RSA", rsa.short_name)

    def test_unknown_oids(self):
        unknown = oid.ObjectIdentifier(value="1.2.3.4")
        self.assertEqual("1.2.3.4", unknown.long_name)
        self.assertEqual("1.2.3.4", unknown.short_name)


if __name__ == '__main__':
    unittest.main()



#
# Cryptographic Message Syntax (CMS)
#
# ASN.1 source from:
# http://tools.ietf.org/html/rfc5652
#
from base64 import b64decode
from smime.crypto.asn1 import oid, types
from smime.crypto.asn1 import cms_common


class SignedData(types.Sequence):
    """
    SignedData ::= SEQUENCE {
      version CMSVersion,
      digestAlgorithms DigestAlgorithmIdentifiers,
      encapContentInfo EncapsulatedContentInfo,
      certificates [0] IMPLICIT CertificateSet OPTIONAL,
      crls [1] IMPLICIT CertificateRevocationLists OPTIONAL,
      signerInfos SignerInfos }
    """
    components = (
        (types.Component('version', cms_common.CMSVersion)),
        (types.Component('digestAlgorithms', cms_common.DigestAlgorithmIdentifiers)),
        (types.Component('encapContentInfo', cms_common.EncapsulatedContentInfo)),
        (types.Component('certificates', cms_common.CertificateSet.implicit(0), optional=True)),
        (types.Component('crls', cms_common.CertificateRevocationLists.implicit(1), optional=True)),
        (types.Component('signerInfos', cms_common.SignerInfos))
        )


class EnvelopedData(types.Sequence):
    """
    EnvelopedData ::= SEQUENCE {
      version CMSVersion,
      originatorInfo [0] IMPLICIT OriginatorInfo OPTIONAL,
      recipientInfos RecipientInfos,
      encryptedContentInfo EncryptedContentInfo,
      unprotectedAttrs [1] IMPLICIT UnprotectedAttributes OPTIONAL }
    """
    components = (
        (types.Component('version', cms_common.CMSVersion)),
        (types.Component('originatorInfo', cms_common.OriginatorInfo.implicit(0), optional=True)),
        (types.Component('recipientInfos', cms_common.RecipientInfos)),
        (types.Component('encryptedContentInfo', cms_common.EncryptedContentInfo)),
        (types.Component('unprotectedAttrs', cms_common.UnprotectedAttributes.implicit(1), optional=True))
        )


class DigestedData(types.Sequence):
    """
    DigestedData ::= SEQUENCE {
      version CMSVersion,
      digestAlgorithm DigestAlgorithmIdentifier,
      encapContentInfo EncapsulatedContentInfo,
      digest Digest }
    """
    components = (
        (types.Component('version', cms_common.CMSVersion)),
        (types.Component('digestAlgorithm', cms_common.DigestAlgorithmIdentifier)),
        (types.Component('encapContentInfo', cms_common.EncapsulatedContentInfo)),
        (types.Component('digest', cms_common.Digest))
        )


class EncryptedData(types.Sequence):
    """
    EncryptedData ::= SEQUENCE {
      version CMSVersion,
      encryptedContentInfo EncryptedContentInfo,
      unprotectedAttrs [1] IMPLICIT UnprotectedAttributes OPTIONAL }
    """
    components = (
        (types.Component('version', cms_common.CMSVersion)),
        (types.Component('encryptedContentInfo', cms_common.EncryptedContentInfo)),
        (types.Component('unprotectedAttrs', cms_common.UnprotectedAttributes.implicit(1), optional=True))
        )


class AuthenticatedData(types.Sequence):
    """
    AuthenticatedData ::= SEQUENCE {
      version CMSVersion,
      originatorInfo [0] IMPLICIT OriginatorInfo OPTIONAL,
      recipientInfos RecipientInfos,
      macAlgorithm MessageAuthenticationCodeAlgorithm,
      digestAlgorithm [1] DigestAlgorithmIdentifier OPTIONAL,
      encapContentInfo EncapsulatedContentInfo,
      authAttrs [2] IMPLICIT AuthAttributes OPTIONAL,
      mac MessageAuthenticationCode,
      unauthAttrs [3] IMPLICIT UnauthAttributes OPTIONAL }
    """
    components = (
        (types.Component('version', cms_common.CMSVersion)),
        (types.Component('originatorInfo', cms_common.OriginatorInfo.implicit(0), optional=True)),
        (types.Component('recipientInfos', cms_common.RecipientInfos)),
        (types.Component('macAlgorithm', cms_common.MessageAuthenticationCodeAlgorithm)),
        (types.Component('digestAlgorithm', cms_common.DigestAlgorithmIdentifier.implicit(1), optional=True)),
        (types.Component('encapContentInfo', cms_common.EncapsulatedContentInfo)),
        (types.Component('authAttrs', cms_common.AuthAttributes.implicit(2), optional=True)),
        (types.Component('mac', cms_common.MessageAuthenticationCode)),
        (types.Component('unauthAttrs', cms_common.UnauthAttributes.implicit(3), optional=True))
        )


_CONTENT_TYPE_DICT = {
    oid.ID_DATA: types.OctetString,
    oid.ID_SIGNED_DATA: SignedData,
    oid.ID_ENVELOPED_DATA: EnvelopedData,
    oid.ID_DIGESTED_DATA: DigestedData,
    oid.ID_ENCRYPTED_DATA: EncryptedData,
    oid.ID_AUTHENTICATED_DATA: AuthenticatedData
    }


class ContentInfo(types.Sequence):
    components = (
        (types.Component('contentType', cms_common.ContentType)),
        (types.Component('content', types.Any.explicit(0), optional=True,
                         defined_by='contentType', lookup=_CONTENT_TYPE_DICT))
        )


def from_string(der, strict_der=True):
    """Read a DER-encoded message from a string.

    Args:
        der: a DER encoded message
        strict_der: if False, tolerate some non-fatal DER errors.

    Returns:
        ContentInfo object.

    Raises:
        smime.crypto.ASN1Error: a block was invalid
        IOError: the file could not be read.
    """
    return ContentInfo.decode(der, strict=strict_der)


def from_file(der_file, strict_der=True):
    """Read a DER-encoded message from a file.

    Args:
        der_file: a DER encoded file.
        strict_der: if False, tolerate some non-fatal DER errors.

    Returns:
        ContentInfo object.

    Raises:
        smime.crypto.error.ASN1Error: a block was invalid
        IOError: the file could not be read.
    """
    with open(der_file, 'rb') as f:
        return from_string(f.read(), strict_der=strict_der)


#!/usr/bin/env python

import unittest

from smime.crypto.asn1 import type_test_base
from smime.crypto.asn1 import cms_common


class LabeledIntegerTest(type_test_base.TypeTestBase):
    class TestLabeledInteger(cms_common.LabeledInteger):
        labels = (('foo', 1), ('bar', 2))

    asn1_type = TestLabeledInteger
    repeated = False
    keyed = False
    initializers = (
        # performed below
        )
    bad_initializers = (
        ('baz', ValueError),
        )
    encode_test_vectors = (
        (0, "020100"),
        ('foo', "020101"),
        ('bar', "020102")
        )
    bad_encodings = (
        # Same as Integer
        )
    bad_strict_encodings = (
        # Same as Integer
        )

    def test_initializers(self):
        foo = self.TestLabeledInteger('foo')
        o1 = self.asn1_type(value=foo)
        self.assertEqual(o1.value, 1)
        bar = self.TestLabeledInteger('bar')
        o2 = self.asn1_type(value=bar)
        self.assertEqual(o2.value, 2)
        bar = self.TestLabeledInteger(3)
        o2 = self.asn1_type(value=bar)
        self.assertEqual(o2.value, 3)

    def test_print(self):
        foo = self.TestLabeledInteger(1)
        self.assertEqual(str(foo), 'foo')


class EnumeratedTest(type_test_base.TypeTestBase):
    class Enumerated(cms_common.Enumerated):
        labels = (('foo', 1), ('bar', 2))

    asn1_type = Enumerated
    repeated = False
    keyed = False
    initializers = (
        # performed below
        )
    bad_initializers = (
        ('baz', TypeError),
        )
    encode_test_vectors = (
        ('foo', "0a0101"),
        ('bar', "0a0102")
        )
    bad_encodings = (
        # Same as Integer
        )
    bad_strict_encodings = (
        # Same as Integer
        )

    def test_initializers(self):
        foo = self.Enumerated('foo')
        self.assertEqual(foo.value, 1)
        bar = self.Enumerated('bar')
        self.assertEqual(bar.value, 2)

    def test_print(self):
        foo = self.Enumerated('foo')
        self.assertEqual(str(foo), 'foo')


if __name__ == '__main__':
    unittest.main()


"""Status codes are bad, but True/False is not expressive enough.

Consider a cryptographic signature verification method verify(data, sig) that
returns 1 for valid signatures, 0 for invalid signatures, and -1 to signal some
specific error. This can easily lead to insecure usage such as
if verify(data, sig):
    # do stuff on success

Or, here's another example, borrowed from real code:
r, s = asn1_decode(sig)  # raises ASN1Error
return verify_sig(data, r, s) # returns True/False

A caller may obviously be interested in distinguishing a decoding error from a
signature computation error - but why is a computation error False while a
decoding error is an exception? What other exceptions might this code raise?
This is a nightmare for the caller to handle.

Therefore, methods in the crypto package that verify a property return True
when verification succeeds and raise an exception on any error. This minimises
the risk of uncaught errors, allows to provide information for callers that care
about the specific failure reason, and makes failure handling easy for callers
that do not care:

try:
    verify(myargs)
except MyError:
    # handle specific error here
    return
except VerifyError:
    # verify failed, we don't care why
    return
# do more stuff on success here

Returning True is strictly speaking not needed but simplifies testing.
We provide a defensive returns_true_or_raises wrapper for ensuring this
behaviour: callers of methods decorated with @returns_true_or_raises can be
certain that the _only_ value the method returns is True - it never returns
None, or False, or [], or anything else.
"""

import functools


class Error(Exception):
    """Exceptions raised by the crypto subpackage."""
    pass


class UnsupportedAlgorithmError(Error):
    """An algorithm is not implemented or supported."""
    pass


class VerifyError(Error):
    """Some expected property of the input cannot be verified.

    The property either verifiably does not hold, or cannot be conclusively
    verified. Domain-specific verification errors inherit from this class.
    """
    pass


class ConsistencyError(VerifyError):
    """There is a (cryptographic) inconsistency in the data."""
    pass


class ProofError(VerifyError):
    """A cryptographic proof is not valid.

    This error does not necessarily indicate that the sought property does not
    hold but rather that the given data is insufficient for verifying the
    desired property.
    """
    pass


# TODO(ekasper): TBD if this hierarchy is appropriate.
class EncodingError(Error):
    """Encoding/decoding error.

    Inputs cannot be serialized, or serialized data cannot be parsed.
    """
    pass


class ASN1Error(EncodingError):
    """An ASN1 object cannot be encoded or decoded."""
    pass


class ASN1TagError(ASN1Error):
    """ASN1 tag mismatch."""
    pass


class UnknownASN1TypeError(ASN1Error):
    """An OID does not map to a known ASN.1 type."""
    pass

class ASN1IllegalCharacter(ASN1Error):
    """String contains illegal character."""
    def __init__(self, message, string, index, *args):
        self.message = message
        self.string = string
        self.index = index
        super(ASN1Error, self).__init__(message, *args)

    def __str__(self):
        return "%s (string: %s, character: %s, index: %d)" % (self.message,
                                                              self.string,
                                                              self.string[
                                                                    self.index],
                                                              self.index)

class IncompleteChainError(VerifyError):
    """A certificate is missing from the chain"""
    pass

class SignatureError(VerifyError):
    """A public-key signature does not verify."""
    pass


class UnsupportedVersionError(Error):
    """The version of the data structure is unknown."""
    pass


def returns_true_or_raises(f):
    """A safety net.

    Decorator for functions that are only allowed to return True or raise
    an exception.

    Args:
        f: A function whose only expected return value is True.

    Returns:
        A wrapped functions whose guaranteed only return value is True.
    """
    @functools.wraps(f)
    def wrapped(*args, **kwargs):
        ret = f(*args, **kwargs)
        if ret is not True:
            raise RuntimeError("Unexpected return value %r" % ret)
        return True
    return wrapped



"""ASN.1 tagging."""

from smime.crypto import error


UNIVERSAL = 0x00
APPLICATION = 0x40
CONTEXT_SPECIFIC = 0x80
PRIVATE = 0xc0
PRIMITIVE = 0x00
CONSTRUCTED = 0x20

# Constants for better readability.
IMPLICIT, EXPLICIT = range(2)


class Tag(object):
    """An ASN.1 tag."""
    _CLASS_MASK = 0xc0
    _ENCODING_MASK = 0x20
    _NUMBER_MASK = 0x1f
    _HIGH = 0x1f
    _FULL_SUB_OCTET = 0x7f
    _LAST_OCTET = 0x80

    def __init__(self, number, tag_class, encoding):
        """ASN.1 tag.

        Initialize a tag from its number, class and encoding.

        Args:
            number: the numeric value of the tag.
            tag_class: must be one of UNIVERSAL, APPLICATION, CONTEXT_SPECIFIC
                or PRIVATE.
            encoding: must be one of PRIMITIVE or CONSTRUCTED.

        Raises:
            ValueError: invalid initializers.
        """
        if tag_class not in (UNIVERSAL, APPLICATION, CONTEXT_SPECIFIC, PRIVATE):
            raise ValueError("Invalid tag class %s" % tag_class)
        if encoding not in (PRIMITIVE, CONSTRUCTED):
            raise ValueError("Invalid encoding %s" % encoding)

        # Public just for lightweight access. Do not modify directly.
        self.number = number
        self.tag_class = tag_class
        self.encoding = encoding
        if number <= 30:
            self.value = chr(tag_class | encoding | number)
        else:
            res = [tag_class | encoding | self._HIGH]
            tmp = []
            while number > 0:
                tmp.append((number & self._FULL_SUB_OCTET) | self._LAST_OCTET)
                number >>= 7
            tmp[0] -= self._LAST_OCTET
            tmp.reverse()
            res += tmp
            self.value = ''.join([chr(byte) for byte in res])

    def __repr__(self):
        return ("%s(%r, %r, %r)" % (self.__class__.__name__, self.number,
                                    self.tag_class, self.encoding))

    def __str__(self):
        return "[%s %d]" % (self.class_name(), self.number)

    def __len__(self):
        return len(self.value)

    def class_name(self):
        if self.tag_class == UNIVERSAL:
            return "UNIVERSAL"
        elif self.tag_class == APPLICATION:
            return "APPLICATION"
        elif self.tag_class == CONTEXT_SPECIFIC:
            return "CONTEXT-SPECIFIC"
        elif self.tag_class == PRIVATE:
            return "PRIVATE"
        else:
            raise ValueError("Invalid tag class %x" % self.tag_class)

    def __hash__(self):
        return hash(self.value)

    def __eq__(self, other):
        if not isinstance(other, Tag):
            return NotImplemented
        return self.value == other.value

    def __ne__(self, other):
        if not isinstance(other, Tag):
            return NotImplemented
        return self.value != other.value

    @classmethod
    def read(cls, buf):
        """Read from the beginning of a string or buffer.

        Args:
            buf: a binary string or string buffer containing an ASN.1 object.

        Returns:
            an tuple consisting of an instance of the class and the remaining
            buffer/string.
        """

        if not buf:
            raise error.ASN1TagError("Ran out of bytes while decoding")
        tag_bytes = 0
        id_byte = ord(buf[tag_bytes])
        tag_class = id_byte & cls._CLASS_MASK
        encoding = id_byte & cls._ENCODING_MASK
        number = id_byte & cls._NUMBER_MASK
        if number == cls._HIGH:
            number = 0
            tag_bytes += 1
            success = False
            for i in range(1, len(buf)):
                number <<= 7
                id_byte = ord(buf[i])
                number |= (id_byte & cls._FULL_SUB_OCTET)
                tag_bytes += 1
                if id_byte & cls._LAST_OCTET == 0:
                    success = True
                    break
            if not success:
                raise error.ASN1TagError("Ran out of bytes while decoding")
            if tag_bytes - 1 > 5:
                raise error.ASN1TagError("Base 128 integer too large")
            tag_bytes -= 1
        tag = cls(number, tag_class, encoding)
        return tag, buf[tag_bytes + 1:]



"""ASN.1 X509 base components."""

from smime.crypto.asn1 import oid
from smime.crypto.asn1 import print_util
from smime.crypto.asn1 import types


class Version(types.Integer):
    def _encode_value(self):
        return types.encode_int(self._value - 1)

    @classmethod
    def _convert_value(cls, value):
        return int(value) + 1

    @classmethod
    def _decode_value(cls, buf, strict=True):
        return types.decode_int(buf, strict=strict) + 1


class CertificateSerialNumber(types.Integer):
    def __str__(self):
        return print_util.int_to_hex(int(self))


class AlgorithmIdentifier(types.Sequence):
    components = (
        (types.Component("algorithm", oid.ObjectIdentifier)),
        (types.Component("parameters", types.Any, optional=True))
        )


class UniqueIdentifier(types.BitString):
    pass


class SubjectPublicKeyInfo(types.Sequence):
    components = (
        (types.Component("algorithm", AlgorithmIdentifier)),
        (types.Component("subjectPublicKey", types.BitString))
        )



from smime.crypto.asn1 import oid

from Crypto.Cipher import AES

from abc import ABCMeta, abstractmethod


class Algorithm():
    __metaclass__ = ABCMeta

    _cipher = None

    def __init__(self, key, **kwargs):
        self._cipher = self.new(key, **kwargs)

    @abstractmethod
    def new(self, **kwargs):
        return NotImplemented

    @abstractmethod
    def encrypt(self, data):
        return NotImplemented


class AES_CBC(Algorithm):
    block_size = 16

    def __init__(self, key, iv):
        super(AES_CBC, self).__init__(key, iv=iv)

    def new(self, session_key, iv=None):
        return AES.new(session_key, AES.MODE_CBC, iv)

    def encrypt(cls, data):
        padded_data = cls._pad(data, cls.block_size)
        return cls._cipher.encrypt(padded_data)

    def _pad(cls, s, block_size):
        n = block_size - len(s) % block_size
        return s + n * chr(n)


class AES128_CBC(AES_CBC):
    oid = oid.ID_AES128_CBC
    key_size = 16


class AES192_CBC(AES_CBC):
    oid = oid.ID_AES192_CBC
    key_size = 24


class AES256_CBC(AES_CBC):
    oid = oid.ID_AES256_CBC
    key_size = 32



"""Python S/MIME handler."""

__author__ = 'G. B. Versiani'
__license__ = 'Apache License (2.0)'
__version__ = '0.0.1'

from .encrypt import encrypt

__all__ = [encrypt, __author__, __license__, __version__]



from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_v1_5

from smime.crypto import cert
from smime.crypto.asn1 import oid, types


class ASN1_RSAPublicKey(types.Sequence):
    components = (
        (types.Component('modulus', types.Integer)),
        (types.Component('publicExponent', types.Integer))
        )


class RSAPublicKey():
    oid = oid.RSA_ENCRYPTION

    def __init__(self, pubkey):
        x509_certs = cert.certs_from_pem(pubkey)
        self._x509_cert = x509_certs.next()._asn1_cert
        der = self._get_public_rsa_key(self._x509_cert)
        self._key = RSA.importKey(der)
        self._cipher = PKCS1_v1_5.new(self._key)

    def _get_public_rsa_key(self, x509_cert):
        bits = x509_cert['tbsCertificate']['subjectPublicKeyInfo']['subjectPublicKey'].value
        return self._bits_to_bytearray(bits)

    @staticmethod
    def _bits_to_bytearray(s):
        assert len(s) % 8 == 0
        result = []
        while s:
            byte = int(''.join([str(x) for x in s[:8]]), 2)
            result.append(chr(byte))
            s = s[8:]
        return ''.join(result)

    def get_cert(self):
        return self._x509_cert

    def encrypt(self, data):
        return self._cipher.encrypt(data)

    def get_parameters(self):
        return {'n': self._key.key.n, 'e': self._key.key.e}

    def __str__(self):
        parms = self.get_parameters()
        return 'RSAPublicKey(n=%d, e=%d)' % (parms['n'], parms['e'])


#!/usr/bin/env python

import unittest

import os
from tempfile import mkstemp
from smime.crypto import pem


class PemTest(unittest.TestCase):
    BLOB = "helloworld\x00"
    MARKER = "BLOB"
    PEM_BLOB = """-----BEGIN BLOB-----
aGVsbG93b3JsZAA=
-----END BLOB-----\n"""

    def setUp(self):
        self.__tempfiles = []

    def create_temp_file(self, contents=None):
        fd, name = mkstemp()
        self.__tempfiles.append(name)
        if contents:
            os.write(fd, contents)
        os.close(fd)
        return name

    def get_file_contents(self, name):
        with open(name) as f:
            return f.read()

    def tearDown(self):
        for name in self.__tempfiles:
            os.remove(name)


class PemReaderTest(PemTest):
    def __assert_default_read_result(self, results):
        self.assertEqual(1, len(results))
        read_blob, read_marker = results[0]
        self.assertEqual(read_blob, self.BLOB)
        self.assertEqual(read_marker, self.MARKER)

    def test_create_reader_from_file(self):
        name = self.create_temp_file(self.PEM_BLOB)

        reader = pem.PemReader.from_file(name, (self.MARKER,))
        self.assertIsInstance(reader, pem.PemReader)

        results = [r for r in reader]
        reader.close()

        self.__assert_default_read_result(results)

    def test_create_reader_from_string(self):
        reader = pem.PemReader.from_string(self.PEM_BLOB, (self.MARKER,))
        self.assertIsInstance(reader, pem.PemReader)

        results = [r for r in reader]
        reader.close()

        self.__assert_default_read_result(results)

    def test_create_reader_from_file_object(self):
        name = self.create_temp_file(self.PEM_BLOB)
        with open(name) as f:
            reader = pem.PemReader(f, (self.MARKER,))
            results = [r for r in reader]

            self.__assert_default_read_result(results)

    def test_reader_as_context_manager(self):
        name = self.create_temp_file(self.PEM_BLOB)
        f = open(name)
        with pem.PemReader(f, (self.MARKER,)) as reader:
            results = [r for r in reader]
            self.__assert_default_read_result(results)
        self.assertTrue(f.closed)

    def test_reader_reads_all_blobs(self):
        blob1 = "hello"
        marker1 = "BLOB"
        pem1 = "-----BEGIN BLOB-----\naGVsbG8=\n-----END BLOB-----\n"

        blob2 = "world"
        marker2 = "CHUNK"
        pem2 = """-----BEGIN CHUNK-----\nd29ybGQ=\n-----END CHUNK-----\n"""
        reader = pem.PemReader.from_string(pem1 + pem2, (marker2, marker1))

        read_blobs = [b for b in reader]
        self.assertEqual(len(read_blobs), 2)
        self.assertEqual((blob1, marker1), read_blobs[0])
        self.assertEqual((blob2, marker2), read_blobs[1])

    def test_reader_honours_markers(self):
        marker1 = "BLOB"
        blob1 = "hello"
        pem1 = "-----BEGIN BLOB-----\naGVsbG8=\n-----END BLOB-----\n"

        # blob2 = "world"
        pem2 = """-----BEGIN CHUNK-----\nd29ybGQ=\n-----END CHUNK-----\n"""
        reader = pem.PemReader.from_string(pem1 + pem2, (marker1,))

        read_blobs = [b for b in reader]
        self.assertEqual(len(read_blobs), 1)
        self.assertEqual((blob1, marker1), read_blobs[0])

    def test_reader_skips_invalid_blocks(self):
        marker = "BLOB"
        blob1 = "hello"
        pem1 = "-----BEGIN BLOB-----\naGVsbG8=\n-----END BLOB-----\n"

        # blob2 = "world"
        pem2 = """-----BEGIN BLOB-----badbase64^&*\n-----END BLOB-----\n"""
        # Make the middle blob invalid and expect the reader to skip it
        # and resume business as usual with the next blob.
        reader = pem.PemReader.from_string(pem1 + pem2 + pem1, [marker])

        read_blobs = [b for b in reader]
        self.assertEqual(len(read_blobs), 2)
        self.assertEqual((blob1, marker), read_blobs[0])
        self.assertEqual((blob1, marker), read_blobs[1])

    def test_reader_raises_on_invalid_block_in_non_skip_mode(self):
        pem1 = "-----BEGIN BLOB-----\naGVsbG8=\n-----END BLOB-----\n"
        pem2 = """-----BEGIN BLOB-----\nbadbase64^&*\n-----END BLOB-----\n"""
        reader = iter(pem.PemReader.from_string(pem1 + pem2, ("BLOB",),
                                                skip_invalid_blobs=False))

        reader.next()
        self.assertRaises(pem.PemError, reader.next)

    def test_reader_stops_no_valid_blocks(self):
        reader = iter(pem.PemReader.from_string("nothing here", ("BLOB",)))
        self.assertRaises(StopIteration, reader.next)
        # Once more for good measure.
        self.assertRaises(StopIteration, reader.next)

    def test_reader_raises_on_no_valid_block_in_non_skip_mode(self):
        reader = iter(pem.PemReader.from_string("nothing here", ("BLOB",),
                                                skip_invalid_blobs=False))
        self.assertRaises(pem.PemError, reader.next)

    def test_reader_matches_markers(self):
        # blob1 = "hello"
        pem1 = "-----BEGIN BLOB-----\naGVsbG8=\n-----END BLOB-----\n"

        # blob2 = "world"
        pem2 = """-----BEGIN BLOB-----\nbadbase64^&*\n-----END CHUNK-----\n"""
        reader = iter(pem.PemReader.from_string(pem1 + pem2,
                                                ("BLOB", "CHUNK")))
        reader.next()
        self.assertRaises(StopIteration, reader.next)

    def test_reader_reads_all_lines(self):
        blob1 = "hello"
        pem1 = "-----BEGIN BLOB-----\naGV\nsbG8=\n-----END BLOB-----\n"
        reader = pem.PemReader.from_string(pem1, ("BLOB",))
        results = [r for r in reader]
        self.assertEqual(1, len(results))
        read_blob, _ = results[0]
        self.assertEqual(read_blob, blob1)

    # Tests for module methods. The module methods are very light wrappers
    # around PemReader, so we don't repeat all of the tests here.
    def test_from_pem(self):
        self.assertEqual((self.BLOB, self.MARKER),
                         pem.from_pem(self.PEM_BLOB, (self.MARKER,)))
        # from_pem will also take a plain string for the marker
        self.assertEqual((self.BLOB, self.MARKER),
                         pem.from_pem(self.PEM_BLOB, self.MARKER))

    def test_from_pem_file(self):
        name = self.create_temp_file(self.PEM_BLOB)
        self.assertEqual((self.BLOB, self.MARKER),
                         pem.from_pem_file(name, (self.MARKER,)))

    def test_pem_blocks(self):
        gen = pem.pem_blocks(self.PEM_BLOB, (self.MARKER,))
        self.assertEqual((self.BLOB, self.MARKER), gen.next())
        self.assertRaises(StopIteration, gen.next)

    def test_pem_blocks_from_file(self):
        name = self.create_temp_file(self.PEM_BLOB)
        gen = pem.pem_blocks_from_file(name, (self.MARKER,))
        self.assertEqual((self.BLOB, self.MARKER), gen.next())
        self.assertRaises(StopIteration, gen.next)


class PemWriterTest(PemTest):
    def test_create_writer_from_file(self):
        name = self.create_temp_file()

        writer = pem.PemWriter.from_file(name, self.MARKER)
        self.assertIsInstance(writer, pem.PemWriter)

        writer.write(self.BLOB)
        writer.close()
        self.assertEqual(self.PEM_BLOB, self.get_file_contents(name))

    def test_create_writer_from_file_object(self):
        name = self.create_temp_file()
        with open(name, "w+") as f:
            writer = pem.PemWriter(f, self.MARKER)
            writer.write(self.BLOB)
            writer.close()
        self.assertEqual(self.PEM_BLOB, self.get_file_contents(name))

    def test_writer_as_context_manager(self):
        name = self.create_temp_file()
        f = open(name, "w+")
        with pem.PemWriter(f, self.MARKER) as writer:
            writer.write(self.BLOB)
        self.assertTrue(f.closed)
        self.assertEqual(self.PEM_BLOB, self.get_file_contents(name))

    def test_write_to_string(self):
        self.assertEqual(pem.PemWriter.pem_string(self.BLOB, self.MARKER),
                         self.PEM_BLOB)

    def test_write_all_blobs(self):
        name = self.create_temp_file()
        blobs = [self.BLOB]*3

        writer = pem.PemWriter.from_file(name, self.MARKER)
        writer.write_blocks(blobs)
        writer.close()
        self.assertEqual(self.PEM_BLOB * 3, self.get_file_contents(name))

    def test_write_all_blobs_to_string(self):
        blobs = [self.BLOB]*3
        self.assertEqual(pem.PemWriter.blocks_to_pem_string(blobs,
                                                            self.MARKER),
                         self.PEM_BLOB * 3)

    def test_append(self):
        contents = "yadayadayada\n"
        name = self.create_temp_file(contents)

        writer = pem.PemWriter.from_file(name, self.MARKER, append=True)
        writer.write(self.BLOB)
        writer.close()
        self.assertEqual(contents + self.PEM_BLOB,
                         self.get_file_contents(name))

    def test_appends_newline(self):
        contents = "yadayadayada"
        name = self.create_temp_file(contents)

        writer = pem.PemWriter.from_file(name, self.MARKER, append=True)
        writer.write(self.BLOB)
        writer.close()
        self.assertEqual(contents + "\n" + self.PEM_BLOB,
                         self.get_file_contents(name))

    def test_write_long_blob(self):
        long_blob = ("IammorethansixtyfourcharacterslongandsoIshouldbesplit"
                     "acrossmultiplelines")
        encoded_long_blob = """-----BEGIN BLOB-----
SWFtbW9yZXRoYW5zaXh0eWZvdXJjaGFyYWN0ZXJzbG9uZ2FuZHNvSXNob3VsZGJl
c3BsaXRhY3Jvc3NtdWx0aXBsZWxpbmVz
-----END BLOB-----
"""
        self.assertEqual(pem.PemWriter.pem_string(long_blob, ("BLOB",)),
                         encoded_long_blob)

    # Module methods
    def test_to_pem(self):
        self.assertEqual(self.PEM_BLOB,
                         pem.to_pem(self.BLOB, self.MARKER))

    def test_blocks_to_pem(self):
        self.assertEqual(self.PEM_BLOB + self.PEM_BLOB,
                         pem.blocks_to_pem([self.BLOB] * 2, self.MARKER))

    def test_to_pem_file(self):
        name = self.create_temp_file()
        pem.to_pem_file(self.BLOB, name, self.MARKER)
        self.assertEqual(self.PEM_BLOB, self.get_file_contents(name))

    def test_blocks_to_pem_file(self):
        name = self.create_temp_file()
        pem.blocks_to_pem_file([self.BLOB] * 2, name, self.MARKER)
        self.assertEqual(self.PEM_BLOB + self.PEM_BLOB,
                         self.get_file_contents(name))

if __name__ == "__main__":
    unittest.main()



"""ASN.1 X509 specification."""

from smime.crypto.asn1 import types
from smime.crypto.asn1 import x509_common
from smime.crypto.asn1 import x509_extension
from smime.crypto.asn1 import x509_name
from smime.crypto.asn1 import x509_time


class TBSCertificate(types.Sequence):
    components = (
        (types.Component("version", x509_common.Version.explicit(0),
                         default=0)),
        (types.Component("serialNumber", x509_common.CertificateSerialNumber)),
        (types.Component("signature", x509_common.AlgorithmIdentifier)),
        (types.Component("issuer", x509_name.Name)),
        (types.Component("validity", x509_time.Validity)),
        (types.Component("subject", x509_name.Name)),
        (types.Component("subjectPublicKeyInfo",
                         x509_common.SubjectPublicKeyInfo)),
        (types.Component("issuerUniqueID",
                         x509_common.UniqueIdentifier.implicit(1),
                         optional=True)),
        (types.Component("subjectUniqueID",
                         x509_common.UniqueIdentifier.implicit(2),
                         optional=True)),
        (types.Component("extensions",
                         x509_extension.Extensions.explicit(3), optional=True))
        )


class Certificate(types.Sequence):
    components = (
        (types.Component("tbsCertificate", TBSCertificate)),
        (types.Component("signatureAlgorithm", x509_common.AlgorithmIdentifier)),
        (types.Component("signatureValue", types.BitString))
        )



"""Utilities for testing time-sensitive code."""

import contextlib
import os
import time


def get_timezone_environ():
    return os.environ.get("TZ", '')

def set_timezone_environ(new_timezone):
    os.environ["TZ"] = new_timezone
    time.tzset()

@contextlib.contextmanager
def timezone(temp_tz):
    """Sets the timezone, yields, then resets the timezone.

    Args:
        temp_tz: See https://docs.python.org/2/library/time.html#time.tzset
    """
    original_tz = get_timezone_environ()
    set_timezone_environ(temp_tz)
    try:
        yield
    finally:
        set_timezone_environ(original_tz)



"""ASN.1 UTCTime and GeneralizedTime, as understood by RFC 5280."""
import abc
import time

from smime.crypto import error
from smime.crypto.asn1 import tag
from smime.crypto.asn1 import types


class BaseTime(types.ASN1String):
    """Base class for time types."""
    def __init__(self, value=None, serialized_value=None, strict=True):
        super(BaseTime, self).__init__(value=value,
                                       serialized_value=serialized_value,
                                       strict=strict)
        self._gmtime = self._decode_gmtime(strict=strict)
        # This is a lenient "strict": if we were able to decode the time,
        # even if it didn't fully conform to the standard, then we'll allow it.
        # If the time string is garbage then we raise.
        if strict and self._gmtime is None:
            raise error.ASN1Error("Corrupt time: %s" % self._value)

    def gmtime(self):
        """GMT time.

        Returns:
            a time.struct_time struct.

        Raises:
            error.ASN1Error: the ASN.1 string does not represent a valid time.
        """
        if self._gmtime is None:
            raise error.ASN1Error("Corrupt time: %s" % self._value)
        return self._gmtime

    @abc.abstractmethod
    def _decode_gmtime(self, strict):
        pass

    def __str__(self):
        try:
            return time.strftime("%c GMT", self.gmtime())
        except error.ASN1Error:
            return str(self.value)


@types.Universal(23, tag.PRIMITIVE)
class UTCTime(BaseTime):
    """UTCTime, as understood by RFC 5280."""
    # YYMMDDHHMMSSZ
    _ASN1_LENGTH = 13

    # YYMMDDHHMMZ
    _UTC_NO_SECONDS_LENGTH = 11

    # YYMMDDHHMMSS+HHMM
    _UTC_TZ_OFFSET_LENGTH = 17

    # YYMMDDHHMMSS
    _UTC_NO_Z_LENGTH = 12

    def _decode_gmtime(self, strict):
        """GMT time.

        Returns:
            a time.struct_time struct, or None if the string does not represent
                a valid time.
        """
        # From RFC 5280:
        # For the purposes of this profile, UTCTime values MUST be expressed in
        # Greenwich Mean Time (Zulu) and MUST include seconds (i.e., times are
        # YYMMDDHHMMSSZ), even where the number of seconds is zero.  Conforming
        # systems MUST interpret the year field (YY) as follows:
        #
        # Where YY is greater than or equal to 50, the year SHALL be
        # interpreted as 19YY; and
        #
        # Where YY is less than 50, the year SHALL be interpreted as 20YY.
        #
        # In addition, there are a number of older certificates
        # that exclude the seconds, e.g. 0001010000Z and others than use
        # an alternative timezone format 360526194526+0000
        string_time = self.value

        if len(string_time) == self._ASN1_LENGTH and string_time[-1] == "Z":
            format = "%Y%m%d%H%M%S%Z"
        elif (len(string_time) == self._UTC_NO_SECONDS_LENGTH and
              string_time[-1] == "Z"):
            format = "%Y%m%d%H%M%Z"
        elif (len(string_time) == self._UTC_TZ_OFFSET_LENGTH and
              string_time[self._UTC_NO_Z_LENGTH] in ('+','-')):
            # note according to http://docs.python.org/2/library/time.html
            # "%z" is not supported on all platforms.
            #
            # TBD: in next patch, parse this correctly
            #
            # Given that it's very infrequent and non-standard,
            # we'll ignore time zone for now.
            #
            # convert the +HHMM to a timedelta and add to timestruct
            # One could also special case the "+0000" which should be the same
            # as GMT (without DST).
            #
            format = "%Y%m%d%H%M%S%Z"
            string_time = string_time[0:self._ASN1_LENGTH]
        elif (len(string_time) == self._UTC_NO_Z_LENGTH) and not strict:
            string_time += "Z"
            format = "%Y%m%d%H%M%S%Z"
        else:
            return None

        try:
            year = int(string_time[:2])
        except ValueError:
            return None

        if 0 <= year < 50:
            century = "20"
        elif 50 <= year <= 99:
            century = "19"
        else:
            return None

        try:
            # Adding GMT clears the daylight saving flag.
            return time.strptime(century + string_time[:-1] + "GMT", format)
        except ValueError:
            return None


@types.Universal(24, tag.PRIMITIVE)
class GeneralizedTime(BaseTime):
    """Generalized time, as understood by RFC 5280."""
    # YYYYMMDDHHMMSSZ
    _ASN1_LENGTH = 15

    def _decode_gmtime(self, strict):
        """GMT time.

        Returns:
            a time.struct_time struct, or None if the string does not represent
                a valid time.
        """
        # From RFC 5280:
        # For the purposes of this profile, GeneralizedTime values MUST be
        # expressed in Greenwich Mean Time (Zulu) and MUST include seconds
        # (i.e., times are YYYYMMDDHHMMSSZ), even where the number of seconds
        # is zero.  GeneralizedTime values MUST NOT include fractional seconds.
        if len(self._value) != self._ASN1_LENGTH or self._value[-1] != "Z":
            return None
        try:
            # Adding GMT clears the daylight saving flag.
            return time.strptime(self._value[:-1] + "GMT", "%Y%m%d%H%M%S%Z")
        except ValueError:
            return None


class Time(types.Choice):
    print_labels = False
    components = {"utcTime": UTCTime,
                  "generalTime": GeneralizedTime}


class Validity(types.Sequence):
    components = (
        (types.Component("notBefore", Time)),
        (types.Component("notAfter", Time))
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='smime',
    version=__import__('smime').__version__,
    description='Python S/MIME Toolkit',
    long_description=long_description,
    url='https://github.com/balena/python-smime',
    author='G. B. Versiani',
    author_email='guibv@yahoo.com',
    license='Apache License (2.0)',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries',
        'Topic :: Communications :: Email',
        'Topic :: Security :: Cryptography',
    ],
    keywords='smime cryptography email',
    packages=find_packages(exclude=['smime/test', 'smime/crypto/testdata',
        'smime/crypto/tools', '*_test.py']),
    platforms=["all"],
    install_requires=['pycrypto'],
)



#!/usr/bin/env python
# coding=utf-8

import unittest

import time
from smime.crypto import cert
from smime.crypto import error
from smime.crypto.asn1 import oid
from smime.crypto.asn1 import x509_extension as x509_ext
from smime.crypto.asn1 import x509_name
from smime.test import test_config

class CertificateTest(unittest.TestCase):
    _PEM_FILE = "google_cert.pem"

    # Contains 3 certificates
    # C=US/ST=California/L=Mountain View/O=Google Inc/CN=www.google.com
    # C=US/O=Google Inc/CN=Google Internet Authority
    # C=US/O=Equifax/OU=Equifax Secure Certificate Authority
    _PEM_CHAIN_FILE = "google_chain.pem"
    _DER_FILE = "google_cert.der"
    # An X509v1 certificate
    _V1_PEM_FILE = "v1_cert.pem"

    # A old but common (0.5% of all certs as of 2013-10-01) SSL
    # cert that uses a different or older DER format for Boolean
    # values.
    _PEM_MATRIXSSL = "matrixssl_sample.pem"

    # Self-signed cert by marchnetworks.com for embedded systems
    # and uses start date in form of "0001010000Z" (no seconds)
    _PEM_MARCHNETWORKS = "marchnetworks_com.pem"

    # Self-signed cert by subrigo.net for embedded systems
    # and uses a start date in the form of 121214093107+0000
    _PEM_SUBRIGONET = "subrigo_net.pem"

    # Self-signed cert by promise.com (as of 2013-10-16) that
    # is in use by embedded systems.
    #
    # * has a start date in the format of 120703092726-1200
    # * uses a 512-key RSA key
    _PEM_PROMISECOM = "promise_com.pem"

    # This self-signed cert was used to test proper (or
    # improper) handling of UTF-8 characters in CN
    # See  CVE 2009-2408 for more details
    #
    # Mozilla bug480509
    # https://bugzilla.mozilla.org/show_bug.cgi?id=480509
    # Mozilla bug484111
    # https://bugzilla.mozilla.org/show_bug.cgi?id=484111
    # RedHat bug510251
    # https://bugzilla.redhat.com/show_bug.cgi?id=510251
    _PEM_CN_UTF8 = "cn_utf8.pem"

    # A self-signed cert with null characters in various names
    # Misparsing was involved in CVE 2009-2408 (above) and
    # CVE-2013-4248
    _PEM_NULL_CHARS = "null_chars.pem"

    # A certificate with a negative serial number, and, for more fun,
    # an extra leading ff-octet therein.
    _PEM_NEGATIVE_SERIAL = "negative_serial.pem"

    # A certificate with an ECDSA key and signature.
    _PEM_ECDSA = "ecdsa_cert.pem"

    # A certificate with multiple EKU extensions.
    _PEM_MULTIPLE_EKU = "multiple_eku.pem"

    # A certificate with multiple "interesting" SANs.
    _PEM_MULTIPLE_AN = "multiple_an.pem"

    # A certificate with multiple CN attributes.
    _PEM_MULTIPLE_CN = "multiple_cn.pem"

    # A certificate with authority cert issuer and authority cert serial.
    _PEM_AKID = "authority_keyid.pem"

    # A certificate chain with an EV policy.
    _PEM_EV_CHAIN = "ev_chain.pem"
    # EV OID for VeriSign Class 3 Public Primary Certification Authority
    _EV_POLICY_OID = oid.ObjectIdentifier(value="2.16.840.1.113733.1.7.23.6")

    _PEM_MULTIPLE_POLICIES = "multiple_policies.pem"

    # A certificate with a UserNotice containing a VisibleString.
    _PEM_USER_NOTICE = "user_notice.pem"

    # A certificate with an invalid (8-byte) IP address in a SAN.
    _PEM_INVALID_IP = "invalid_ip.pem"

    # A certificate with both kinds of AIA information.
    _PEM_AIA = "aia.pem"

    # A certificate with ASN1 indefinite length encoding.
    _PEM_INDEFINITE_LENGTH = "asn1_indefinite_length_encoding.pem"

    # A certificate with 99991231235959Z expiration date
    _PEM_NOT_WELL_DEFINED_EXPIRATION = "expiration_not_well_defined.pem"

    # A certificate with street address, postal code etc. provided
    _PEM_WITH_ADDRESS = "cert_with_address.pem"

    @property
    def pem_file(self):
        return test_config.get_test_file_path(self._PEM_FILE)

    def get_file(self, filename):
        return test_config.get_test_file_path(filename)

    def cert_from_pem_file(self, filename, strict=True):
        return cert.Certificate.from_pem_file(
            self.get_file(filename), strict_der=strict)

    def test_from_pem_file(self):
        c = self.cert_from_pem_file(self._PEM_FILE)
        self.assertTrue(isinstance(c, cert.Certificate))

    def test_certs_from_pem_file(self):
        certs = list(cert.certs_from_pem_file(self.get_file(
            self._PEM_CHAIN_FILE)))
        self.assertEqual(3, len(certs))
        self.assertTrue(all([isinstance(x, cert.Certificate) for x in certs]))
        self.assertTrue("google.com" in certs[0].print_subject_name())
        self.assertTrue("Google Inc" in certs[1].print_subject_name())
        self.assertTrue("Equifax" in certs[2].print_subject_name())

    def test_from_pem(self):
        with open(self.get_file(self._PEM_FILE)) as f:
            c = cert.Certificate.from_pem(f.read())
        self.assertTrue(isinstance(c, cert.Certificate))

    def test_to_pem(self):
        with open(self.get_file(self._PEM_FILE)) as f:
            c = cert.Certificate.from_pem(f.read())
        # PEM files can and do contain arbitrary additional information,
        # so we can't assert equality with the original contents.
        # Instead, simply check that we can read the newly constructed PEM.
        new_pem = c.to_pem()
        c2 = cert.Certificate.from_pem(new_pem)
        self.assertTrue(c2.is_identical_to(c))

    def test_all_from_pem(self):
        with open(self.get_file(self._PEM_CHAIN_FILE)) as f:
            certs = list(cert.certs_from_pem(f.read()))
        self.assertEqual(3, len(certs))
        self.assertTrue(all([isinstance(x, cert.Certificate) for x in certs]))
        self.assertTrue("google.com" in certs[0].print_subject_name())
        self.assertTrue("Google Inc" in certs[1].print_subject_name())
        self.assertTrue("Equifax" in certs[2].print_subject_name())

    def test_from_der_file(self):
        c = cert.Certificate.from_der_file(self.get_file(self._DER_FILE))
        self.assertTrue(isinstance(c, cert.Certificate))

    def test_from_der(self):
        with open(self.get_file(self._DER_FILE), "rb") as f:
            cert_der = f.read()
            c = cert.Certificate.from_der(cert_der)
        self.assertTrue(isinstance(c, cert.Certificate))
        self.assertEqual(c.to_der(), cert_der)

    def test_invalid_encoding_raises(self):
        self.assertRaises(error.EncodingError, cert.Certificate.from_der,
                          "bogus_der_string")
        self.assertRaises(error.EncodingError, cert.Certificate.from_pem,
                          "bogus_pem_string")

    def test_to_der(self):
        with open(self.get_file(self._DER_FILE), "rb") as f:
            der_string = f.read()
        c = cert.Certificate(der_string)
        self.assertEqual(der_string, c.to_der())

    def test_identical_to_self(self):
        c = self.cert_from_pem_file(self._PEM_FILE)
        self.assertTrue(c.is_identical_to(c))
        self.assertEqual(c, c)

    def test_identical(self):
        c = self.cert_from_pem_file(self._PEM_FILE)
        c2 = self.cert_from_pem_file(self._PEM_FILE)
        self.assertTrue(c.is_identical_to(c2))
        self.assertTrue(c2.is_identical_to(c))
        self.assertEqual(c2, c)

    def test_not_identical(self):
        c = self.cert_from_pem_file(self._PEM_FILE)
        c2 = self.cert_from_pem_file(self._V1_PEM_FILE)
        self.assertFalse(c2.is_identical_to(c))
        self.assertNotEqual(c2, c)
        self.assertNotEqual(c2, "foo")

    def test_hash(self):
        c = self.cert_from_pem_file(self._PEM_FILE)
        c2 = self.cert_from_pem_file(self._PEM_FILE)
        self.assertEqual(hash(c), hash(c))
        self.assertEqual(hash(c), hash(c2))

    def test_parse_matrixssl(self):
        """Test parsing of old MatrixSSL.org sample certificate

        As of 2013-10-01, about 0.5% of all SSL sites use an old
        sample certificate from MatrixSSL.org. It appears it's used
        mostly for various home routers.  Unfortunately it uses a
        non-DER encoding for boolean value: the DER encoding of True
        is 0xFF but this cert uses a BER encoding of 0x01. This causes
        pure DER parsers to break.  This test makes sure we can parse
        this cert without exceptions or errors.
        """
        self.assertRaises(error.ASN1Error,
                          self.cert_from_pem_file, self._PEM_MATRIXSSL)
        c = self.cert_from_pem_file(self._PEM_MATRIXSSL, strict=False)
        issuer = c.print_issuer_name()
        self.assertTrue("MatrixSSL Sample Server" in issuer)

    def test_parse_marchnetworks(self):
        """Test parsing certificates issued by marchnetworks.com."""
        c = self.cert_from_pem_file(self._PEM_MARCHNETWORKS)
        issuer = c.print_issuer_name()
        self.assertTrue("March Networks" in issuer)

        # 0001010000Z
        expected = [2000, 1, 1, 0, 0, 0, 5, 1, 0]
        self.assertEqual(list(c.not_before()), expected)

        # 3001010000Z
        expected = [2030, 1, 1, 0, 0, 0, 1, 1, 0]
        self.assertEqual(list(c.not_after()), expected)

    def test_parse_subrigonet(self):
        """Test parsing certificates issued by subrigo.net

        The certificates issued by subrigo.net (non-root)
        use an start date with time zone.

            Not Before: Dec 14 09:31:07 2012
            Not After : Dec 13 09:31:07 2022 GMT

        """
        c = self.cert_from_pem_file(self._PEM_SUBRIGONET)
        issuer = c.print_issuer_name()
        self.assertTrue("subrigo.net" in issuer)

        # timezone format -- 121214093107+0000
        expected = [2012, 12, 14, 9, 31, 7, 4, 349, 0]
        self.assertEqual(list(c.not_before()), expected)

        # standard format -- 221213093107Z
        expected = [2022, 12, 13, 9, 31, 7, 1, 347, 0]
        self.assertEqual(list(c.not_after()), expected)

    def test_utf8_names(self):
        c = self.cert_from_pem_file(self._PEM_CN_UTF8)
        nameutf8 = "ñeco ñýáěšžěšžřěčíě+ščýáíéřáíÚ"
        unicodename = u"ñeco ñýáěšžěšžřěčíě+ščýáíéřáíÚ"
        # Compare UTF-8 strings directly.
        self.assertEqual(c.print_subject_name(), "CN=" + nameutf8)
        self.assertEqual(c.print_issuer_name(), "CN=" + nameutf8)
        cns = c.subject_common_names()
        self.assertEqual(1, len(cns))
        self.assertEqual(cns[0], nameutf8)
        # Name comparison is unicode-based so decode and compare unicode names.
        # TODO(ekasper): implement proper stringprep-based name comparison
        # and use these test cases there.
        self.assertEqual(cns[0].value.decode("utf8"), unicodename)

    def test_null_chars_in_names(self):
        """Test handling null chars in subject and subject alternative names."""
        c = self.cert_from_pem_file(self._PEM_NULL_CHARS)
        cns = c.subject_common_names()
        self.assertEqual(1, len(cns))
        self.assertEqual("null.python.org\000example.org", cns[0])

        alt_names = c.subject_alternative_names()
        self.assertEqual(len(alt_names), 5)
        self.assertEqual(alt_names[0].component_key(), x509_name.DNS_NAME)
        self.assertEqual(alt_names[0].component_value(),
                         "altnull.python.org\000example.com")
        self.assertEqual(alt_names[1].component_key(), x509_name.RFC822_NAME)
        self.assertEqual(alt_names[1].component_value(),
                         "null@python.org\000user@example.org")
        self.assertEqual(alt_names[2].component_key(),x509_name.URI_NAME)
        self.assertEqual(alt_names[2].component_value(),
                         "http://null.python.org\000http://example.org")

        # the following does not contain nulls.
        self.assertEqual(alt_names[3].component_key(),
                         x509_name.IP_ADDRESS_NAME)
        self.assertEqual(alt_names[3].component_value().as_octets(),
                         (192, 0, 2, 1))
        self.assertEqual(alt_names[4].component_key(),
                         x509_name.IP_ADDRESS_NAME)
        self.assertEqual(alt_names[4].component_value().as_octets(),
                         (32, 1, 13, 184, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1))

    def test_parse_promisecom(self):
        """Test parsing certificates issued by promise.com

        The certificates issued by promise.com (non-root)
        use an start date with time zone (and are 512-bit)

            Not Before: Jun 29 15:32:48 2011
            Not After : Jun 26 15:32:48 2021 GMT
        """

        c = self.cert_from_pem_file(self._PEM_PROMISECOM)
        issuer = c.print_issuer_name()
        self.assertTrue("Promise Technology Inc." in issuer)

        # 110629153248-1200
        expected = [2011,6,29,15,32,48,2,180,0]
        self.assertEqual(list(c.not_before()), expected)

        # 210626153248Z
        expected = [2021,6,26,15,32,48,5,177,0]
        self.assertEqual(list(c.not_after()), expected)

    def test_parse_ecdsa_cert(self):
        c = self.cert_from_pem_file(self._PEM_ECDSA)
        self.assertTrue("kmonos.jp" in c.print_subject_name())
        self.assertEquals(oid.ECDSA_WITH_SHA256, c.signature()["algorithm"])
        self.assertEquals(oid.ECDSA_WITH_SHA256,
                          c.signature_algorithm()["algorithm"])

    def test_print_subject_name(self):
        c = self.cert_from_pem_file(self._PEM_FILE)
        subject = c.print_subject_name()
        # C=US, ST=California, L=Mountain View, O=Google Inc, CN=*.google.com
        self.assertTrue("US" in subject)
        self.assertTrue("California" in subject)
        self.assertTrue("Mountain View" in subject)
        self.assertTrue("Google Inc" in subject)
        self.assertTrue("*.google.com" in subject)

    def test_print_issuer_name(self):
        c = self.cert_from_pem_file(self._PEM_FILE)
        issuer = c.print_issuer_name()
        # Issuer: C=US, O=Google Inc, CN=Google Internet Authority
        self.assertTrue("US" in issuer)
        self.assertTrue("Google Inc" in issuer)
        self.assertTrue("Google Internet Authority" in issuer)

    def test_subject_common_names(self):
        c = self.cert_from_pem_file(self._PEM_FILE)
        cns = c.subject_common_names()
        self.assertEqual(1, len(cns))
        self.assertEqual("*.google.com", cns[0])

    def test_multiple_subject_common_names(self):
        c = self.cert_from_pem_file(self._PEM_MULTIPLE_CN)
        cns = c.subject_common_names()
        self.assertItemsEqual(cns, ["www.rd.io", "rdio.com", "rd.io",
                                    "api.rdio.com", "api.rd.io",
                                    "www.rdio.com"])

    def test_subject_dns_names(self):
        c = self.cert_from_pem_file(self._PEM_FILE)
        dns_names = c.subject_dns_names()
        self.assertEqual(44, len(dns_names))
        self.assertTrue("*.youtube.com" in dns_names)

    def test_subject_ip_addresses(self):
        c = self.cert_from_pem_file(self._PEM_MULTIPLE_AN)
        ips = c.subject_ip_addresses()
        self.assertEqual(1, len(ips))
        self.assertEqual((129, 48, 105, 104), ips[0].as_octets())

    def test_invalid_ip_addresses(self):
        with self.assertRaises(error.ASN1Error) as fail:
            self.cert_from_pem_file(self._PEM_INVALID_IP)

        self.assertIn("00000000ffffff00", str(fail.exception))
        c = self.cert_from_pem_file(self._PEM_INVALID_IP, strict=False)
        ips = c.subject_ip_addresses()
        self.assertEqual(1, len(ips))
        self.assertEqual((0, 0, 0, 0, 255, 255, 255, 0), ips[0].as_octets())

    def test_subject_alternative_names(self):
        cert = self.cert_from_pem_file(self._PEM_MULTIPLE_AN)
        sans = cert.subject_alternative_names()
        self.assertEqual(4, len(sans))

        self.assertEqual(x509_name.DNS_NAME, sans[0].component_key())
        self.assertEqual("spires.wpafb.af.mil", sans[0].component_value())

        self.assertEqual(x509_name.DIRECTORY_NAME, sans[1].component_key())
        self.assertTrue(isinstance(sans[1].component_value(), x509_name.Name),
        sans[1].component_value())

        self.assertEqual(x509_name.IP_ADDRESS_NAME, sans[2].component_key())
        self.assertEqual((129, 48, 105, 104),
        sans[2].component_value().as_octets())

        self.assertEqual(x509_name.URI_NAME, sans[3].component_key())
        self.assertEqual("spires.wpafb.af.mil", sans[3].component_value())

    def test_no_alternative_names(self):
        c = cert.Certificate.from_pem_file(self.get_file(self._V1_PEM_FILE))
        self.assertEqual(0, len(c.subject_alternative_names()))
        self.assertEqual(0, len(c.subject_dns_names()))
        self.assertEqual(0, len(c.subject_ip_addresses()))

    def test_validity(self):
        certs = list(cert.certs_from_pem_file(
            self.get_file(self._PEM_CHAIN_FILE)))
        self.assertEqual(3, len(certs))
        # notBefore: Sat Aug 22 16:41:51 1998 GMT
        # notAfter: Wed Aug 22 16:41:51 2018 GMT
        c = certs[2]
        # These two will start failing in 2018.
        self.assertTrue(c.is_temporally_valid_now())
        self.assertFalse(c.is_expired())

        self.assertFalse(c.is_not_yet_valid())

        # Aug 22 16:41:51 2018
        self.assertTrue(c.is_temporally_valid_at(time.gmtime(1534956111)))
        # Aug 22 16:41:52 2018
        self.assertFalse(c.is_temporally_valid_at(time.gmtime(1534956112)))

        # Aug 22 16:41:50 1998
        self.assertFalse(c.is_temporally_valid_at(time.gmtime(903804110)))
        # Aug 22 16:41:51 1998
        self.assertTrue(c.is_temporally_valid_at(time.gmtime(903804111)))

    def test_basic_constraints(self):
        certs = list(cert.certs_from_pem_file(
            self.get_file(self._PEM_CHAIN_FILE)))
        self.assertFalse(certs[0].basic_constraint_ca())
        self.assertTrue(certs[1].basic_constraint_ca())
        self.assertIsNone(certs[0].basic_constraint_path_length())
        self.assertEqual(0, certs[1].basic_constraint_path_length())

    def test_version(self):
        c = self.cert_from_pem_file(self._PEM_FILE)
        self.assertEqual(3, c.version())

    def test_issuer_common_name(self):
        c = self.cert_from_pem_file(self._PEM_FILE)
        icn = c.issuer_common_name()
        self.assertIn("Google Internet Authority", icn[0].value)
        self.assertEqual(len(icn), 1)

    def test_issuer_country_name(self):
        c = self.cert_from_pem_file(self._PEM_FILE)
        icn = c.issuer_country_name()
        self.assertIn("US", icn)
        self.assertEqual(len(icn), 1)

    def test_subject_organization_name(self):
        c = self.cert_from_pem_file(self._PEM_FILE)
        icn = c.subject_organization_name()
        self.assertIn("Google Inc", icn)
        self.assertEqual(len(icn), 1)

    def test_subject_street_address(self):
        c = self.cert_from_pem_file(self._PEM_WITH_ADDRESS)
        address = c.subject_street_address()
        self.assertIn("CQ Mail Centre", address)
        self.assertIn("Building 19", address)

    def test_subject_locality_name(self):
        c = self.cert_from_pem_file(self._PEM_WITH_ADDRESS)
        locality_name = c.subject_locality_name()
        self.assertIn("Rockhampton", locality_name)

    def test_subject_state_or_province(self):
        c = self.cert_from_pem_file(self._PEM_WITH_ADDRESS)
        state_or_province = c.subject_state_or_province_name()
        self.assertIn("Queensland", state_or_province)

    def test_subject_postal_code(self):
        c = self.cert_from_pem_file(self._PEM_WITH_ADDRESS)
        postal_code = c.subject_postal_code()
        self.assertIn("4702", postal_code)

    def test_serial_number(self):
        c = self.cert_from_pem_file(self._PEM_FILE)
        self.assertEqual(454887626504608315115709, c.serial_number())

    def test_negative_serial_number(self):
        # Fails because of the leading ff-octet.
        self.assertRaises(error.ASN1Error, self.cert_from_pem_file,
            self._PEM_NEGATIVE_SERIAL)
        c = self.cert_from_pem_file(self._PEM_NEGATIVE_SERIAL,
                                    strict=False)
        self.assertEqual(-218943125988803304701934765446014018,
                          c.serial_number())

    def test_v1_cert(self):
        c = self.cert_from_pem_file(self._V1_PEM_FILE)
        self.assertEqual(1, c.version())
        self.assertIsNone(c.basic_constraint_ca())

    def test_fingerprint(self):
        c = cert.Certificate.from_der_file(self.get_file(self._DER_FILE))
        self.assertEqual(c.fingerprint().encode("hex"),
                         "570fe2e3bfee986ed4a158aed8770f2e21614659")
        self.assertEqual(c.fingerprint("sha1").encode("hex"),
                         "570fe2e3bfee986ed4a158aed8770f2e21614659")
        self.assertEqual(c.fingerprint("sha256").encode("hex"),
                         "6d4106b4544e9e5e7a0924ee86a577ffefaadae8b8dad73413a7"
                         "d874747a81d1")

    def test_key_usage(self):
        c = cert.Certificate.from_pem_file(self.get_file(self._PEM_FILE))
        self.assertTrue(c.key_usage(x509_ext.KeyUsage.DIGITAL_SIGNATURE))

        certs = [c for c in cert.certs_from_pem_file(self.get_file(
            self._PEM_CHAIN_FILE))]
        # This leaf cert does not have a KeyUsage extension.
        self.assertEqual([], certs[0].key_usages())
        self.assertIsNone(certs[0].key_usage(
            x509_ext.KeyUsage.DIGITAL_SIGNATURE))

        # The second cert has keyCertSign and cRLSign.
        self.assertIsNotNone(certs[1].key_usage(
            x509_ext.KeyUsage.DIGITAL_SIGNATURE))
        self.assertFalse(certs[1].key_usage(
            x509_ext.KeyUsage.DIGITAL_SIGNATURE))
        self.assertTrue(certs[1].key_usage(x509_ext.KeyUsage.KEY_CERT_SIGN))
        self.assertTrue(certs[1].key_usage(x509_ext.KeyUsage.CRL_SIGN))
        self.assertItemsEqual([x509_ext.KeyUsage.KEY_CERT_SIGN,
                               x509_ext.KeyUsage.CRL_SIGN],
                              certs[1].key_usages())

    def test_extended_key_usage(self):
        certs = [c for c in cert.certs_from_pem_file(self.get_file(
            self._PEM_CHAIN_FILE))]
        self.assertTrue(certs[0].extended_key_usage(oid.ID_KP_SERVER_AUTH))
        self.assertIsNotNone(
            certs[0].extended_key_usage(oid.ID_KP_CODE_SIGNING))
        self.assertFalse(certs[0].extended_key_usage(oid.ID_KP_CODE_SIGNING))
        self.assertItemsEqual([oid.ID_KP_SERVER_AUTH, oid.ID_KP_CLIENT_AUTH],
                              certs[0].extended_key_usages())

        # EKU is normally only found in leaf certs.
        self.assertIsNone(certs[1].extended_key_usage(oid.ID_KP_SERVER_AUTH))
        self.assertEqual([], certs[1].extended_key_usages())

    def test_multiple_extensions(self):
        self.assertRaises(error.ASN1Error, cert.Certificate.from_pem_file,
                          self.get_file(self._PEM_MULTIPLE_EKU))

        c = cert.Certificate.from_pem_file(self.get_file(self._PEM_MULTIPLE_EKU),
                                           strict_der=False)
        self.assertTrue("www.m-budget-mobile-abo.ch" in c.subject_common_names())
        self.assertRaises(cert.CertificateError, c.extended_key_usages)

    def test_key_identifiers(self):
        certs = [c for c in cert.certs_from_pem_file(self.get_file(
            self._PEM_CHAIN_FILE))]

        self.assertEqual("\x12\x4a\x06\x24\x28\xc4\x18\xa5\x63\x0b\x41\x6e\x95"
                         "\xbf\x72\xb5\x3e\x1b\x8e\x8f",
                         certs[0].subject_key_identifier())

        self.assertEqual("\xbf\xc0\x30\xeb\xf5\x43\x11\x3e\x67\xba\x9e\x91\xfb"
                         "\xfc\x6a\xda\xe3\x6b\x12\x24",
                         certs[0].authority_key_identifier())

        self.assertIsNone(certs[0].authority_key_identifier(
            identifier_type=x509_ext.AUTHORITY_CERT_ISSUER))
        self.assertIsNone(certs[0].authority_key_identifier(
            identifier_type=x509_ext.AUTHORITY_CERT_SERIAL_NUMBER))

        self.assertEqual(certs[0].authority_key_identifier(),
                         certs[1].subject_key_identifier())

        c = self.cert_from_pem_file(self._PEM_AKID)

        cert_issuers = c.authority_key_identifier(
            identifier_type=x509_ext.AUTHORITY_CERT_ISSUER)
        self.assertEqual(1, len(cert_issuers))

        # A DirectoryName.
        cert_issuer = cert_issuers[0]
        self.assertEqual(x509_name.DIRECTORY_NAME, cert_issuer.component_key())
        self.assertEqual(["KISA RootCA 1"],
                         cert_issuer.component_value().attributes(
                             oid.ID_AT_COMMON_NAME))
        self.assertEqual(10119, c.authority_key_identifier(
            identifier_type=x509_ext.AUTHORITY_CERT_SERIAL_NUMBER))

    def test_policies(self):
        certs = [c for c in cert.certs_from_pem_file(self.get_file(
            self._PEM_EV_CHAIN))]
        ev_cert = certs[0]
        policies = ev_cert.policies()
        self.assertEqual(1, len(policies))

        self.assertTrue(ev_cert.has_policy(self._EV_POLICY_OID))
        self.assertFalse(ev_cert.has_policy(oid.ANY_POLICY))

        policy = ev_cert.policy(self._EV_POLICY_OID)

        qualifiers = policy[x509_ext.POLICY_QUALIFIERS]
        self.assertEqual(1, len(qualifiers))

        qualifier = qualifiers[0]
        self.assertEqual(oid.ID_QT_CPS, qualifier[x509_ext.POLICY_QUALIFIER_ID])
        # CPS location is an Any(IA5String).
        self.assertEqual("https://www.verisign.com/cps",
                         qualifier[x509_ext.QUALIFIER].decoded_value)

        any_cert = certs[1]
        policies = any_cert.policies()
        self.assertEqual(1, len(policies))

        self.assertFalse(any_cert.has_policy(self._EV_POLICY_OID))
        self.assertTrue(any_cert.has_policy(oid.ANY_POLICY))

        policy = ev_cert.policy(self._EV_POLICY_OID)

        qualifiers = policy[x509_ext.POLICY_QUALIFIERS]
        self.assertEqual(1, len(qualifiers))

        qualifier = qualifiers[0]
        self.assertEqual(oid.ID_QT_CPS, qualifier[x509_ext.POLICY_QUALIFIER_ID])
        # CPS location is an IA5String.
        self.assertEqual("https://www.verisign.com/cps",
                         qualifier[x509_ext.QUALIFIER].decoded_value)

        no_policy_cert = certs[2]
        self.assertEqual(0, len(no_policy_cert.policies()))
        self.assertFalse(no_policy_cert.has_policy(self._EV_POLICY_OID))
        self.assertFalse(no_policy_cert.has_policy(oid.ANY_POLICY))

    def test_multiple_policies(self):
        c = self.cert_from_pem_file(self._PEM_MULTIPLE_POLICIES)
        policies = c.policies()
        self.assertEqual(2, len(policies))
        self.assertTrue(c.has_policy(oid.ObjectIdentifier(
            value="1.3.6.1.4.1.6449.1.2.2.7")))
        self.assertTrue(c.has_policy(oid.ObjectIdentifier(
            value="2.23.140.1.2.1")))
        self.assertFalse(c.has_policy(oid.ANY_POLICY))

    def test_user_notice(self):
        c = self.cert_from_pem_file(self._PEM_USER_NOTICE)
        policies = c.policies()
        self.assertEqual(1, len(policies))

        qualifiers = policies[0][x509_ext.POLICY_QUALIFIERS]
        self.assertEqual(2, len(qualifiers))

        qualifier = qualifiers[0]
        self.assertEqual(oid.ID_QT_UNOTICE,
                         qualifier[x509_ext.POLICY_QUALIFIER_ID])

        qualifier = qualifier[x509_ext.QUALIFIER].decoded_value
        self.assertIsNone(qualifier[x509_ext.NOTICE_REF])
        expected_text = ("For more details, please visit our website "
                         "https://www.cybertrust.ne.jp .")
        explicit_text = qualifier[x509_ext.EXPLICIT_TEXT].component_value()
        self.assertEqual(expected_text, explicit_text)

    def test_crl_distribution_points(self):
        c = self.cert_from_pem_file(self._PEM_FILE)
        crls = c.crl_distribution_points()
        self.assertEqual(1, len(crls))
        crl = crls[0]

        # Optional components, not present.
        self.assertIsNone(crl[x509_ext.REASONS])
        self.assertIsNone(crl[x509_ext.CRL_ISSUER])

        # This is the prevalent form of CRL distribution points.
        dist_points = crl[x509_ext.DISTRIBUTION_POINT]
        self.assertEqual(x509_ext.FULL_NAME, dist_points.component_key())
        self.assertEqual(1, len(dist_points.component_value()))

        # A GeneralName URI.
        dist_point = dist_points.component_value()[0]
        self.assertEqual("http://www.gstatic.com/GoogleInternetAuthority/"
                         "GoogleInternetAuthority.crl",
                         dist_point[x509_name.URI_NAME])

    def test_aia(self):
        c = self.cert_from_pem_file(self._PEM_AIA)

        ca_issuers = c.ca_issuers()
        self.assertEqual(1, len(ca_issuers))

        # A GeneralName URI.
        self.assertEqual("http://pki.google.com/GIAG2.crt",
                         ca_issuers[0][x509_name.URI_NAME])

        ocsp = c.ocsp_responders()
        self.assertEqual(1, len(ocsp))

        self.assertEqual("http://clients1.google.com/ocsp",
                         ocsp[0][x509_name.URI_NAME])

        # Cert has CA issuers but no OCSP responders.
        c = self.cert_from_pem_file(self._PEM_FILE)
        self.assertItemsEqual([], c.ocsp_responders())

    def test_is_self_signed_root(self):
        c = self.cert_from_pem_file(self._PEM_SUBRIGONET)
        self.assertTrue(c.is_self_signed())

    def test_is_self_signed_leaf(self):
        c = self.cert_from_pem_file(self._PEM_AIA)
        self.assertFalse(c.is_self_signed())

    def test_get_extensions(self):
        c = self.cert_from_pem_file(self._PEM_AIA)
        extensions = c.get_extensions()
        extensions_oids = [extension['extnID'] for extension in extensions]
        self.assertItemsEqual((oid.ID_CE_EXT_KEY_USAGE,
                               oid.ID_CE_SUBJECT_ALT_NAME,
                               oid.ID_PE_AUTHORITY_INFO_ACCESS,
                               oid.ID_CE_SUBJECT_KEY_IDENTIFIER,
                               oid.ID_CE_BASIC_CONSTRAINTS,
                               oid.ID_CE_AUTHORITY_KEY_IDENTIFIER,
                               oid.ID_CE_CERTIFICATE_POLICIES,
                               oid.ID_CE_CRL_DISTRIBUTION_POINTS),
                              extensions_oids)

    def test_indefinite_encoding(self):
        self.assertRaises(error.ASN1Error, self.cert_from_pem_file,
                          self._PEM_INDEFINITE_LENGTH)
        c = self.cert_from_pem_file(self._PEM_INDEFINITE_LENGTH, strict=False)
        issuer = c.print_issuer_name()
        self.assertTrue("VeriSign Class 1 CA" in issuer)

    def test_expiration_not_well_defined(self):
        c = self.cert_from_pem_file(self._PEM_NOT_WELL_DEFINED_EXPIRATION)
        self.assertFalse(c.is_not_after_well_defined())
        # Make sure that certificate with regular expiration date return true
        c = self.cert_from_pem_file(self._PEM_AIA)
        self.assertTrue(c.is_not_after_well_defined())


if __name__ == "__main__":
    unittest.main()




"""X509 Certificate API."""

import collections
import hashlib
import time

from smime.crypto import error
from smime.crypto import pem
from smime.crypto.asn1 import oid
from smime.crypto.asn1 import x509
from smime.crypto.asn1 import x509_extension as x509_ext
from smime.crypto.asn1 import x509_name


class CertificateError(error.Error):
    """Certificate has errors."""
    pass


class Certificate(object):
    """X509 certificates."""
    PEM_MARKERS = ("CERTIFICATE",)

    def __init__(self, der_string, strict_der=True):
        """Initialize from a DER string.

        Args:
            der_string: a binary string containing the DER-encoded
                certificate.
            strict_der: if False, tolerate some non-fatal DER errors.

        Raises:
            error.ASN1Error: invalid encoding.
        """
        # ASN.1 errors fall through.
        self._asn1_cert = x509.Certificate.decode(der_string,
                                                  strict=strict_der)
        # The general philosophy here is that a certificate decoded in
        # strict mode should never raise CertificateErrors later on in the
        # code. Strict mode already catches corrupt extensions, to the extent
        # that their IDs are recognized; in addition, we have to ensure that
        # no extension appears more than once.
        # TODO(ekasper): move this check to the Extensions class.
        if strict_der and self._has_multiple_extension_values():
            raise error.ASN1Error("Multiple extensions")
        # Certs are primarily used as read-only objects so we cache the DER
        # encoding. If any public setters or other methods modifying the
        # contents of the certificate are ever added to this class, they must
        # invalidate the cached encoding.

    def __repr__(self):
        # This prints the full ASN1 representation. Useful for debugging.
        return repr(self._asn1_cert)

    def __str__(self):
        return self._asn1_cert.human_readable(label=self.__class__.__name__)

    def __eq__(self, other):
        if isinstance(other, type(self)):
            return self.is_identical_to(other)
        else:
            return NotImplemented

    def __ne__(self, other):
        are_equal = self.__eq__(other)

        if are_equal is NotImplemented:
            return NotImplemented
        else:
            return not are_equal

    def __hash__(self):
        return hash(self.fingerprint())

    @classmethod
    def from_pem(cls, pem_string, strict_der=True):
        """Read a single PEM-encoded certificate from a string.

        Args:
            pem_string: the certificate string.
            strict_der: if False, tolerate some non-fatal DER errors.

        Returns:
            a Certificate object.

        Raises:
            smime.crypto.pem.PemError, smime.crypto.error.ASN1Error: the string
            does not contain a valid PEM certificate.
        """
        der_cert, _ = pem.from_pem(pem_string, cls.PEM_MARKERS)
        return cls.from_der(der_cert, strict_der=strict_der)

    def _has_multiple_extension_values(self):
        """Returns true if any extension appears more than once."""
        extns = self._asn1_cert["tbsCertificate"]["extensions"] or []
        extn_value_count = collections.Counter([e["extnID"] for e in extns])
        return any([c > 1 for c in list(extn_value_count.values())])

    def _get_decoded_extension_value(self, extn_id):
        """Get the decoded value of an extension.

        Args:
            extn_id: extension OID.

        Returns:
            the decoded extension value matching an extn_id, or None.

        Raises:
            CertificateError: corrupt extension, or multiple extensions
                matching the given OID.
        """
        decoded_extn_values = self._get_decoded_extension_values(extn_id)
        if not decoded_extn_values:
            return None
        if len(decoded_extn_values) > 1:
            # TODO(ekasper): could refine this to only raise when the multiple
            # extension values are conflicting.
            raise CertificateError("Multiple extension values")
        return decoded_extn_values[0]

    def _get_decoded_extension_values(self, extn_id):
        """Get all decoded values matching an extension ID.
        Args:
            extn_id: extension OID.

        Returns:
            a list of decoded extension values matching an extn_id
            (i.e., tolerates multiple extensions).

        Raises:
            CertificateError: corrupt extension.
        """
        extns = self._asn1_cert["tbsCertificate"]["extensions"] or []
        extn_values = [e["extnValue"] for e in extns if e["extnID"] == extn_id]
        if not extn_values:
            return []
        decoded_values = [e.decoded_value for e in extn_values]
        if any([val is None for val in decoded_values]):
            raise CertificateError("Corrupt or unrecognized extension: %s"
                                   % extn_id)
        return decoded_values

    @classmethod
    def from_der(cls, der_string, strict_der=True):
        """Read a single DER-encoded certificate from a string.

        This is just an alias to __init__ to match from_pem().

        Args:
            der_string: the certificate string.
            strict_der: if False, tolerate some non-fatal DER errors.

        Returns:
            a Certificate object.

        Raises:
            smime.crypto.error.ASN1Error: the string does not contain a valid
            DER certificate.
        """
        return cls(der_string, strict_der=strict_der)

    @classmethod
    def from_pem_file(cls, pem_file, strict_der=True):
        """Read a single PEM-encoded certificate from a file.

        Args:
            pem_file: the certificate file.
            strict_der: if False, tolerate some non-fatal DER errors.

        Returns:
            a Certificate object
        Raises:
            smime.crypto.pem.PemError, smime.crypto.error.ASN1Error: the file does not
            contain a valid PEM certificate
            IOError: the file could not be read,
        """
        der_cert, _ = pem.from_pem_file(pem_file, cls.PEM_MARKERS)
        return cls.from_der(der_cert, strict_der=strict_der)

    @classmethod
    def from_der_file(cls, der_file, strict_der=True):
        """Read a single DER-encoded certificate from a file.

        Args:
            der_file: the certificate file.
            strict_der: if False, tolerate some non-fatal DER errors.

        Returns:
            a Certificate object.

        Raises:
            smime.crypto.error.ASN1Error: the file does not contain a valid DER
            certificate
            IOError: the file could not be read.
        """
        with open(der_file, "rb") as der_cert_file:
            return cls.from_der(der_cert_file.read(), strict_der=strict_der)

    def to_der(self):
        """Get the DER-encoding of the certificate."""
        return self._asn1_cert.encode()

    def to_pem(self):
        """Get the PEM-encoding of the certificate."""
        return pem.to_pem(self._asn1_cert.encode(), self.PEM_MARKERS[0])

    def is_identical_to(self, other_cert):
        """Returns True if this certificate is identical to |other_cert|."""
        return self.to_der() == other_cert.to_der()

    def to_asn1(self):
        """Get a copy of the ASN.1 representation of the certificate."""
        return x509.Certificate.decode(self._asn1_cert.encode())

    def get_extensions(self):
        """Get all extensions.

        Returns:
            a list of extensions.
        """
        return self._asn1_cert["tbsCertificate"]["extensions"] or []

    def version(self):
        """Get the version.

        Returns:
            an integral value of the version (i.e., V1 is 0).
        """
        return self._asn1_cert["tbsCertificate"]["version"]

    def issuer_common_name(self):
        """Get the common names of the issuer.

        Returns:
            an list of common name strings

        Raises:
            Corrupt issuer common name attribute.
        """
        try:
            return self._asn1_cert["tbsCertificate"]["issuer"].attributes(
                    oid.ID_AT_COMMON_NAME)
        except error.ASN1Error:
            raise CertificateError("Corrupt issuer common name attribute")

    def issuer_country_name(self):
        """Get issuer country name.

        Returns:
            an list of issuer country names.

        Raises:
            Corrupt issuer country name attribute.
        """
        try:
            return self._asn1_cert["tbsCertificate"]["issuer"].attributes(
                    oid.ID_AT_COUNTRY_NAME)
        except error.ASN1Error:
            raise CertificateError("Corrupt issuer country name attribute.")

    def subject_common_names(self):
        """Get the common names of the subject.

        Returns:
            a list of common name strings ("CN" attribute values in the
            certificate's "subject" field).

        Raises:
            CertificateError: corrupt CN attribute.
        """
        try:
            return self._asn1_cert["tbsCertificate"]["subject"].attributes(
                oid.ID_AT_COMMON_NAME)
        except error.ASN1Error:
            raise CertificateError("Corrupt common name attribute")

    def subject_organization_name(self):
        """Get subject organization name.

        Returns:
            a list of subject organization names.

        Raises:
            CertificateError: corrupt subject organization name attribute.
        """
        try:
            return self._asn1_cert["tbsCertificate"]["subject"].attributes(
                    oid.ID_AT_ORGANIZATION_NAME)
        except error.ASN1Error:
            raise CertificateError("Corrupt subject organization name "
                                   "attribute.")

    def subject_street_address(self):
        """Get subject street address.

        Returns:
            a list of subject street addresses.

        Raises:
            CertificateError: corrupt subject street adress attribute.
        """
        try:
            return self._asn1_cert["tbsCertificate"]["subject"].attributes(
                    oid.ID_AT_STREET_ADDRESS)
        except error.ASN1Error:
            raise CertificateError("Corrupt subject street address attribute.")

    def subject_locality_name(self):
        """Get subject locality name.

        Returns:
            a list of subject locality names.

        Raises:
            CertificateError: corrupt subject locality names attribute.
        """
        try:
            return self._asn1_cert["tbsCertificate"]["subject"].attributes(
                    oid.ID_AT_LOCALITY_NAME)
        except error.ASN1Error:
            raise CertificateError("Corrupt subject locality name attribute")

    def subject_state_or_province_name(self):
        """Get subject state or province name.

        Returns:
            a list of subject state or province names.

        Raises:
            CertificateError: corrupt subject state or province names attribute.
        """
        try:
            return self._asn1_cert["tbsCertificate"]["subject"].attributes(
                    oid.ID_AT_STATE_OR_PROVINCE_NAME)
        except error.ASN1Error:
            raise CertificateError("Corrupt subject state or province name "
                                   "attribute.")

    def subject_postal_code(self):
        """Get subject postal code.

        Returns:
            a list of subject postal codes.

        Raises:
            CertificateError: corrupt subject postal code attribute.
        """
        try:
            return self._asn1_cert["tbsCertificate"]["subject"].attributes(
                    oid.ID_AT_POSTAL_CODE)
        except error.ASN1Error:
            raise CertificateError("Corrupt subject postal code attribute.")

    def subject_organizational_unit_name(self):
        """Get subject organizational unit name.

        Returns:
            a list of subject organizational unit names.

        Raises:
            CertificateError: corrupt subject organizational unit name attribute
        """
        try:
            return self._asn1_cert["tbsCertificate"]["subject"].attributes(
                    oid.ID_AT_ORGANIZATIONAL_UNIT_NAME)
        except error.ASN1Error:
            raise CertificateError("Corrupt subject organizational unit name "
                                   "attribute.")

    def subject_alternative_names(self):
        """Get a list of subjectAlternativeNames extension values.

        Returns:
            a list of subject alternative names. Each element in the list
            is an ASN.1 GeneralName. If multiple SAN extensions are present
            (non-strict mode), returns a list of all names found (possibly
            with duplicates).

            Since each GeneralName is a choice, its contents can be inspected
            as follows:

            sans = cert.subject_alternative_names()
            for san in sans:
                # The type of the alternative name; one of
                # x509_name.OTHER_NAME
                # x509_name.RFC822_NAME
                # x509_name.X400_ADDRESS_NAME
                # x509_name.DIRECTORY_NAME
                # x509_name.EDI_PARTY_NAME
                # x509_name.URI_NAME
                # x509_name.IP_ADDRESS_NAME
                # x509_name.REGISTERED_ID_NAME
                san_type = san.component_key()

                # The corresponding ASN.1 value:
                san_value = san.component_value()
                ...

        Raises:
            CertificateError: corrupt extension.
        """
        sans = self._get_decoded_extension_values(oid.ID_CE_SUBJECT_ALT_NAME)
        return sum([list(san) for san in sans], [])

    def subject(self):
        """Returns list of subject field values in fashion similar to
        subject_alternative_names fashion.
        """
        subject = self._asn1_cert["tbsCertificate"]["subject"]
        return [(sub['type'], sub['value'])
                for sub in subject.flatten()]

    def issuer(self):
        """Returns list of issuer field values in fashion similar to
        subject method fashion.
        """
        issuer = self._asn1_cert["tbsCertificate"]["issuer"]
        return [(iss['type'], iss['value'])
                for iss in issuer.flatten()]

    def _get_subject_alt_names_by_type(self, san_type):
        """Returns the SAN extension values corresponding to |san_type|"""
        # A certificate should only have one SAN extension but we can't rely on
        # this (in non-strict mode), so we return everything we find.
        sans = self._get_decoded_extension_values(oid.ID_CE_SUBJECT_ALT_NAME)
        ret = []
        for san in sans:
            ret += [c.component_value() for c in san
                    if c.component_key() == san_type]
        return ret

    # DNS names and IP addresses are relevant to HTTPS server identification.
    # Other types of alternative names *should* normally be ignored by web
    # browsers but may be relevant for other services.
    # See http://tools.ietf.org/html/rfc6125
    def subject_dns_names(self):
        """Get the dnsNames in the subjectAlternativeNames extension.

        Returns:
            a list of DNS names in the subjectAlternativeNames (SAN) extension.
            If multiple SAN extensions are present (non-strict mode), returns
            a list of all DNS names found (possibly with duplicates).

        Raises:
            CertificateError: corrupt extension.
        """
        return self._get_subject_alt_names_by_type(x509_name.DNS_NAME)

    def subject_ip_addresses(self):
        """Get the ipAddresses in the subjectAlternativeNames extension.

        Returns:
            a list of IPAddress entries in the subjectAlternativeNames (SAN)
            extension. If multiple SAN extensions are present (non-strict mode),
            returns a list of all IP addresses found (possibly with duplicates).

        Raises:
            CertificateError: corrupt extension.
        """
        return self._get_subject_alt_names_by_type(x509_name.IP_ADDRESS_NAME)

    def print_subject_name(self):
        """Get a human readable string of the subject name attributes."""
        return (self._asn1_cert["tbsCertificate"]["subject"].
                human_readable(wrap=0))

    def print_issuer_name(self):
        """Get a human readable string of the issuer name attributes."""
        return (self._asn1_cert["tbsCertificate"]["issuer"].
                human_readable(wrap=0))

    def serial_number(self):
        """Get the serial number.

        While the serial number is an integer, it could be very large.
        RFC 5280 specification states is should not be longer than 20 octets,
        and also states users SHOULD be prepared to gracefully
        handle non-conforming certificates.

        Returns:
           An ASN.1 Integer.

        """
        return self._asn1_cert["tbsCertificate"]["serialNumber"]

    def signature(self):
        """Get TBSCertificate signature.

        Returns:
            an ASN.1 BitString.
        """
        return self._asn1_cert["tbsCertificate"]["signature"]

    def signature_algorithm(self):
        """Get Certificate signature algorithm.

        Returns:
            an AlgorithmIdentifier
        """
        return self._asn1_cert["signatureAlgorithm"]

    def basic_constraint_ca(self):
        """Get the BasicConstraints CA value.

        Returns:
            an ASN.1 Boolean, or None.

        Raises:
            CertificateError: corrupt extension, or multiple extensions.
        """
        # CertificateErrors fall through.
        bc = self._get_decoded_extension_value(oid.ID_CE_BASIC_CONSTRAINTS)
        if bc is None:
            return None

        return bc["cA"]

    def basic_constraint_path_length(self):
        """Get the BasicConstraints pathLenConstraint value.

        Returns:
            an ASN.1 Integer, or None.

        Raises:
            CertificateError: corrupt extension, or multiple extensions.
        """
        bc = self._get_decoded_extension_value(oid.ID_CE_BASIC_CONSTRAINTS)
        if bc is None:
            return None

        return bc["pathLenConstraint"]

    def not_before(self):
        """Get a time.struct_time representing the notBefore in UTC time.

        Returns:
            a time.struct_time object.

        Raises:
            CertificateError: corrupt notBefore value.
        """
        return (self._asn1_cert["tbsCertificate"]["validity"]["notBefore"].
                component_value().gmtime())

    def not_after(self):
        """Get a time.struct_time representing the notAfter in UTC time.

        Returns:
            a time.struct_time object.

        Raises:
            CertificateError: corrupt notAfter value.
        """
        return (self._asn1_cert["tbsCertificate"]["validity"]["notAfter"].
                component_value().gmtime())

    def is_not_after_well_defined(self):
        """Checks if notAfter field is well defined.

        RFC5280 specifies that to indicate that certificate has no well-defined
        expiration date notAfter should be set to 99991231235959Z
        generalized time.

        Returns:
            True if not after is well defined, False otherwise
        """
        not_after = self._asn1_cert["tbsCertificate"]["validity"]["notAfter"]
        if "generalTime" in not_after.value:
            not_after = not_after.value["generalTime"]
            if not_after.value == '99991231235959Z':
                return False
        return True

    def is_temporally_valid_now(self):
        """Determine whether notBefore <= now <= notAfter.

        Returns:
            True or False.

        Raises:
            CertificateError: corrupt time.
        """
        return self.is_temporally_valid_at(time.gmtime())

    def is_expired(self):
        """Is certificate notAfter in the past?

        Returns:
            True or False.

        Raises:
            CertificateError: corrupt time.
        """
        now = time.gmtime()
        return now > self.not_after()

    def is_not_yet_valid(self):
        """Is certificate notBefore in the future?

        Returns:
            True or False.

        Raises:
            CertificateError: corrupt time.
        """
        now = time.gmtime()
        return now < self.not_before()

    def is_temporally_valid_at(self, gmtime):
        """Is certificate valid at the given moment?

        Args:
            gmtime: a struct_time GMT time.

        Returns:
            True or False.

        Raises:
            CertificateError: corrupt time.
        """
        return self.not_before() <= gmtime <= self.not_after()

    def is_self_signed(self):
        """Is self signed?

        Returns:
            True or False.
        """
        return (self._asn1_cert["tbsCertificate"]["issuer"] ==
                self._asn1_cert["tbsCertificate"]["subject"])

    def fingerprint(self, hashfunc="sha1"):
        """Get the certificate fingerprint.

        Args:
            hashfunc: name of a hash function. Algorithms always present are
                'md5', 'sha1', 'sha224', 'sha256', 'sha384', and 'sha512'.
        Returns:
            a (binary) hash digest of the DER encoding.
        """
        h = hashlib.new(hashfunc)
        h.update(self._asn1_cert.encode())
        return h.digest()

    def key_hash(self, hashfunc="sha1"):
        """Get the certificate's public key hash.

        Args:
            hashfunc: name of a hash function. Algorithms always present are
                'md5', 'sha1', 'sha224', 'sha256', 'sha384', and 'sha512'.
        Returns:
            a (binary) hash digest of the public key.
        """
        h = hashlib.new(hashfunc)
        h.update(
            self._asn1_cert["tbsCertificate"]["subjectPublicKeyInfo"].encode())
        return h.digest()

    def key_usage(self, key_usage):
        """Whether the certificate has the given key usage asserted.

        Args:
            key_usage: the usage; one of
            x509_extension.KeyUsage.DIGITAL_SIGNATURE
            x509_extension.KeyUsage.NON_REPUDIATION
            x509_extension.KeyUsage.KEY_ENCIPHERMENT
            x509_extension.KeyUsage.DATA_ENCIPHERMENT
            x509_extension.KeyUsage.KEY_AGREEMENT
            x509_extension.KeyUsage.KEY_CERT_SIGN
            x509_extension.KeyUsage.CRL_SIGN
            x509_extension.KeyUsage.ENCIPHER_ONLY
            x509_extension.KeyUsage.DECIPHER_ONLY

        Returns:
            True: the key usage is asserted.
            False: the key usage extension is present but the specified
                usage is not asserted.
            None: the key usage extension is not present.

        Raises:
            CertificateError: corrupt key usage extension, or multiple
                extension values.
        """
        ku = self._get_decoded_extension_value(oid.ID_CE_KEY_USAGE)
        if ku is None:
            return None
        return ku.has_bit_set(key_usage.value)

    def key_usages(self):
        """List the asserted key usages.

        Returns:
            A list of key usages asserted in the certificate (or an empty
                list if the extension is not present).

        Raises:
            CertificateError: corrupt key usage extension, or multiple
                extension values.
        """
        ku = self._get_decoded_extension_value(oid.ID_CE_KEY_USAGE)
        return ku.bits_set() if ku else []

    def extended_key_usage(self, ext_key_usage):
        """Whether the certificate has the given extended key usage asserted.

        Args:
            ext_key_usage: the object identifier of the usage, e.g.,
                smime.crypto.asn1.oid.ID_KP_SERVER_AUTH.

        Returns:
            True: the extended key usage is asserted.
            False: the extended key usage extension is present but the
                specified usage is not asserted.
            None: the extended key usage extension is not present.

        Raises:
            CertificateError: corrupt key usage extension, or multiple
                extension values.
        """
        eku = self._get_decoded_extension_value(oid.ID_CE_EXT_KEY_USAGE)
        if eku is None:
            return None
        return ext_key_usage in eku

    def extended_key_usages(self):
        """List the asserted extended key usages.

        Returns:
            A sequence of extended key usage identifiers asserted in the
            certificate (or an empty list if the extension is not present).

        Raises:
            CertificateError: corrupt key usage extension, or multiple
                extension values.
        """
        return self._get_decoded_extension_value(oid.ID_CE_EXT_KEY_USAGE) or []

    def subject_key_identifier(self):
        """Get the subject key identifier.

        Returns:
            An x509_extension.KeyIdentifier (ASN.1 OctetString) holding the
            value of the subject key identifier, or None if the subject key
            identifier extension is not present.
        Raises:
            CertificateError: corrupt extension, or multiple extension values.
        """
        return self._get_decoded_extension_value(
            oid.ID_CE_SUBJECT_KEY_IDENTIFIER)

    def authority_key_identifier(self, identifier_type=x509_ext.KEY_IDENTIFIER):
        """Get the authority key identifier of the given type.

        Args:
            identifier_type: the identifier component to fetch, one of
              x509_extension.KEY_IDENTIFIER,
              x509_extension.AUTHORITY_CERT_ISSUER,
              x509_extension.AUTHORITY_CERT_SERIAL_NUMBER.

        Returns:
            the identifier component of the appropriate type, or None if the
            component/extension is not present. The types are
              x509_extension.KEY_IDENTIFIER: x509_extension.KeyIdentifier
                  (an OCTET STRING),
              x509_extension.AUTHORITY_CERT_ISSUER: x509_name.GeneralNames,
              x509_extension.AUTHORITY_CERT_SERIAL_NUMBER:
                  x509_common.CertificateSerialNumber.

        Raises:
            CertificateError: corrupt extension, or multiple extension values.
        """
        akid = self._get_decoded_extension_value(
            oid.ID_CE_AUTHORITY_KEY_IDENTIFIER)

        return akid[identifier_type] if akid else None

    def policies(self):
        """List certificate policies.

        Returns:
            a list of certificate policies, or an empty list if the extension is
            not present.

        Raises:
            CertificateError: corrupt extension, or multiple extension values.
        """
        policies = self._get_decoded_extension_value(
            oid.ID_CE_CERTIFICATE_POLICIES)
        return policies or []

    def policy(self, policy_oid):
        """Find a policy with the given OID.

        Returns:
            The matching policy, or None.

        Raises:
            CertificateError: corrupt extension, or multiple extension values.
        """
        policies = self.policies()

        if not policies:
            return None

        matching = [p for p in policies
                    if p[x509_ext.POLICY_IDENTIFIER] == policy_oid]
        if not matching:
            return None

        # TODO(ekasper): make this exception fire earlier in strict mode.
        if len(matching) > 1:
            raise CertificateError("Policy %s asserted more than once" %
                                   policy_oid)
        return matching[0]

    def has_policy(self, policy_oid):
        """Whether the certificate has a given policy.

        Returns:
            True if the given policy is asserted, False otherwise.

        Raises:
            CertificateError: corrupt extension, or multiple extension values.
        """
        return self.policy(policy_oid) is not None

    def crl_distribution_points(self):
        """List CRL distribution points.

        Returns:
            a list of DistributionPoint entries.

        Raises:
            CertificateError: corrupt extension, or multiple extension values.
        """
        return (self._get_decoded_extension_value(
            oid.ID_CE_CRL_DISTRIBUTION_POINTS) or [])

    def ca_issuers(self):
        """List CA issuers from the Authority Information Access extension.

        Returns:
            a list of CA issuers (as ASN.1 GeneralNames).

        Raises:
            CertificateError: corrupt extension, or multiple extension values.
        """
        aia = self._get_decoded_extension_value(oid.ID_PE_AUTHORITY_INFO_ACCESS)
        if aia is None:
            return []
        return [a[x509_ext.ACCESS_LOCATION] for a in aia
                if a[x509_ext.ACCESS_METHOD] == oid.ID_AD_CA_ISSUERS]

    def ocsp_responders(self):
        """List OCSP responders from the Authority Information Access extension.

        Returns:
            a list of OCSP responders (as ASN.1 GeneralNames).

        Raises:
            CertificateError: corrupt extension, or multiple extension values.
        """
        aia = self._get_decoded_extension_value(oid.ID_PE_AUTHORITY_INFO_ACCESS)
        if aia is None:
            return []
        return [a[x509_ext.ACCESS_LOCATION] for a in aia
                if a[x509_ext.ACCESS_METHOD] == oid.ID_AD_OCSP]

    def embedded_sct_list(self):
        """Get the encoded list of embedded timestamps

        Returns:
            bytes representing a TLS encoded SignedCertificateTimestampList,
            or None if not present

        Raises:
            CertificateError: corrupt extension, or multiple extension values.
        """
        signed_certificate_timestamp_list = (
                self._get_decoded_extension_value(oid.CT_EMBEDDED_SCT_LIST))
        if signed_certificate_timestamp_list is None:
            return None
        return signed_certificate_timestamp_list.value

    def has_extension(self, extn_id):
        """Check if certificate contains a given extnsion.

        Args:
            extn_id: extension OID.

        Returns:
            True or False

        Raises:
            CertificateError: corrupt extension, or multiple extension values.
        """
        return self._get_decoded_extension_value(extn_id) is not None

def certs_from_pem(pem_string, skip_invalid_blobs=False, strict_der=True):
    """Read multiple PEM-encoded certificates from a string.

    Args:
        pem_string: the certificate string
        skip_invalid_blobs: if False, invalid PEM blobs cause a PemError.
            If True, invalid blobs are skipped. In non-skip mode, an
            immediate StopIteration before any valid blocks are found also
            causes a PemError exception.
        strict_der: if False, tolerate some non-fatal DER errors.

    Yields:
        Certificate objects.

    Raises:
        smime.crypto.pem.PemError, smime.crypto.ASN1Error: a block was invalid
        IOError: the file could not be read.
    """
    for der_cert, _ in pem.pem_blocks(pem_string, Certificate.PEM_MARKERS,
                                      skip_invalid_blobs=skip_invalid_blobs):
        try:
            yield Certificate.from_der(der_cert, strict_der=strict_der)
        except error.ASN1Error:
            if not skip_invalid_blobs:
                raise


def certs_from_pem_file(pem_file, skip_invalid_blobs=False, strict_der=True):
    """Read multiple PEM-encoded certificates from a file.

    Args:
        pem_file: the certificate file.
        skip_invalid_blobs: if False, invalid PEM blobs cause a PemError.
            If True, invalid blobs are skipped. In non-skip mode, an
            immediate StopIteration before any valid blocks are found also
            causes a PemError exception.
        strict_der: if False, tolerate some non-fatal DER errors.

    Yields:
        Certificate objects.

    Raises:
        smime.crypto.pem.PemError, smime.crypto.error.ASN1Error:
            a block was invalid
        IOError: the file could not be read.
    """
    for der_cert, _ in pem.pem_blocks_from_file(
        pem_file, Certificate.PEM_MARKERS,
        skip_invalid_blobs=skip_invalid_blobs):
        try:
            yield Certificate.from_der(der_cert, strict_der=strict_der)
        except error.ASN1Error:
            if not skip_invalid_blobs:
                raise



#!/usr/bin/env python

import unittest
from smime.crypto.asn1 import print_util

class PrintUtilTest(unittest.TestCase):
    def test_bits_to_hex(self):
        bit_array = [0,1,1,0,1,0,1,1,1,0]
        self.assertEqual("01:ae", print_util.bits_to_hex(bit_array))
        self.assertEqual("01ae", print_util.bits_to_hex(bit_array, delimiter=""))
        self.assertEqual("", print_util.bits_to_hex(""))

    def test_bytes_to_hex(self):
        byte_array = "\x01\xae"
        self.assertEqual("01:ae", print_util.bytes_to_hex(byte_array))
        self.assertEqual("01ae", print_util.bytes_to_hex(byte_array, delimiter=""))
        self.assertEqual("", print_util.bytes_to_hex(""))

    def test_int_to_hex(self):
        integer = 1234 # 0x4d2
        self.assertEqual("04:d2", print_util.int_to_hex(integer))
        self.assertEqual("04d2", print_util.int_to_hex(integer, delimiter=""))
        negative_integer = -1234
        self.assertEqual(" -:04:d2", print_util.int_to_hex(negative_integer))

    def test_wrap_lines(self):
        long_multiline_string = "hello\nworld"
        self.assertEqual(["hel", "lo", "wor", "ld"],
                         print_util.wrap_lines(long_multiline_string, 3))

    def test_wrap_lines_no_wrap(self):
        long_multiline_string = "hello\nworld"
        self.assertEqual(["hello", "world"],
                         print_util.wrap_lines(long_multiline_string, 0))

    def test_append_lines_appends(self):
        buf = ["hello"]
        lines = ["beautiful", "world"]
        # "hellobeautiful" is more than 10 characters long
        print_util.append_lines(lines, 20, buf)
        self.assertEqual(["hellobeautiful", "world"], buf)

    def test_append_lines_honours_wrap(self):
        buf = ["hello"]
        lines = ["beautiful", "world"]
        # "hellobeautiful" is more than 10 characters long
        print_util.append_lines(lines, 10, buf)
        self.assertEqual(["hello", "beautiful", "world"], buf)

if __name__ == "__main__":
    unittest.main()
